# -*- coding: utf-8 -*-
# pylint: disable=invalid-name,line-too-long,protected-access,too-many-locals
# pylint: disable=too-many-lines
"""
Primitive ``ctypes`` prototypes for the NAG Library `blgm` Chapter.
"""

# NAG Copyright 2017-2019.

from ..base import utils

lm_formula = utils._EngineState._get_symbol('g22yaf')
lm_formula.restype = None
lm_formula.argtypes = (
    utils._BYREF_C_PTR, # Input/Output, hform.
    utils._BYREF_STRING, # Input, formula(len=*).
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of formula.
)

lm_describe_data = utils._EngineState._get_symbol('g22ybf')
lm_describe_data.restype = None
lm_describe_data.argtypes = (
    utils._BYREF_C_PTR, # Input/Output, hddesc.
    utils._BYREF_INT, # Input, nobs.
    utils._BYREF_INT, # Input, nvar.
    utils._EngineIntArrayType.c_type, # Input, levels[nvar].
    utils._BYREF_INT, # Input, lvnames.
    utils._EngineCharArrayType.c_type, # Input, vnames(len=*)[lvnames].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vnames.
)

lm_design_matrix = utils._EngineState._get_symbol('g22ycf')
lm_design_matrix.restype = None
lm_design_matrix.argtypes = (
    utils._BYREF_C_PTR, # Input, hform.
    utils._BYREF_C_PTR, # Input, hddesc.
    utils._EngineFloat64ArrayType.c_type, # Input, dat[lddat, sddat].
    utils._BYREF_INT, # Input, lddat.
    utils._BYREF_INT, # Input, sddat.
    utils._BYREF_C_PTR, # Input/Output, hxdesc.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, sdx].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, sdx.
    utils._BYREF_INT, # Output, mx.
    utils._BYREF_INT, # Input/Output, ifail.
)

lm_submodel = utils._EngineState._get_symbol('g22ydf')
lm_submodel.restype = None
lm_submodel.argtypes = (
    utils._BYREF_STRING, # Input, what(len=*).
    utils._BYREF_C_PTR, # Input, hform.
    utils._BYREF_C_PTR, # Input, hxdesc.
    utils._BYREF_STRING, # Output, intcpt(len=*).
    utils._BYREF_INT, # Output, ip.
    utils._BYREF_INT, # Input, lisx.
    utils._EngineIntArrayType.c_type, # Output, isx[lisx].
    utils._BYREF_INT, # Input, lplab.
    utils._EngineCharArrayType.c_type, # Output, plab(len=*)[lplab].
    utils._BYREF_INT, # Input, lvinfo.
    utils._EngineIntArrayType.c_type, # Output, vinfo[lvinfo].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of what.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of intcpt.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of plab.
)

handle_free = utils._EngineState._get_symbol('g22zaf')
handle_free.restype = None
handle_free.argtypes = (
    utils._BYREF_C_PTR, # Input/Output, handle.
    utils._BYREF_INT, # Input/Output, ifail.
)

optset = utils._EngineState._get_symbol('g22zmf')
optset.restype = None
optset.argtypes = (
    utils._BYREF_C_PTR, # Input, handle.
    utils._BYREF_STRING, # Input, optstr(len=*).
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
)

optget = utils._EngineState._get_symbol('g22znf')
optget.restype = None
optget.argtypes = (
    utils._BYREF_C_PTR, # Input, handle.
    utils._BYREF_STRING, # Input, optstr(len=*).
    utils._BYREF_INT, # Output, ivalue.
    utils._BYREF_FLOAT64, # Output, rvalue.
    utils._BYREF_STRING, # Output, cvalue(len=*).
    utils._BYREF_INT, # Output, optype.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cvalue.
)
