# -*- coding: utf-8 -*-
# pylint: disable=invalid-name,line-too-long,protected-access,too-many-locals
# pylint: disable=too-many-lines
"""
Primitive ``ctypes`` prototypes for the NAG Library `stat` Chapter.
"""

# NAG Copyright 2017-2019.

from ..base import utils

summary_2var = utils._EngineState._get_symbol('g01abf')
summary_2var.restype = None
summary_2var.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x1[n].
    utils._EngineFloat64ArrayType.c_type, # Input, x2[n].
    utils._BYREF_INT, # Input/Output, iwt.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, wt[n].
    utils._EngineFloat64ArrayType.c_type, # Output, res[13].
    utils._BYREF_INT, # Input/Output, ifail.
)

summary_freq = utils._EngineState._get_symbol('g01adf')
summary_freq.restype = None
summary_freq.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._EngineFloat64ArrayType.c_type, # Input, x[k].
    utils._EngineIntArrayType.c_type, # Input, ifreq[k].
    utils._BYREF_FLOAT64, # Output, xmean.
    utils._BYREF_FLOAT64, # Output, s2.
    utils._BYREF_FLOAT64, # Output, s3.
    utils._BYREF_FLOAT64, # Output, s4.
    utils._BYREF_INT, # Output, n.
    utils._BYREF_INT, # Input/Output, ifail.
)

frequency_table = utils._EngineState._get_symbol('g01aef')
frequency_table.restype = None
frequency_table.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, k.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._BYREF_INT, # Input, iclass.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, cb[k].
    utils._EngineIntArrayType.c_type, # Output, ifreq[k].
    utils._BYREF_FLOAT64, # Output, xmin.
    utils._BYREF_FLOAT64, # Output, xmax.
    utils._BYREF_INT, # Input/Output, ifail.
)

contingency_table = utils._EngineState._get_symbol('g01aff')
contingency_table.restype = None
contingency_table.argtypes = (
    utils._BYREF_INT, # Input, ldnob.
    utils._BYREF_INT, # Input, ldpred.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineIntArrayType.c_type, # Input/Output, nobs[ldnob, n].
    utils._BYREF_INT, # Input/Output, num.
    utils._EngineFloat64ArrayType.c_type, # Output, pred[ldpred, n].
    utils._BYREF_FLOAT64, # Output, chis.
    utils._EngineFloat64ArrayType.c_type, # Output, p[21].
    utils._BYREF_INT, # Output, npos.
    utils._BYREF_INT, # Output, ndf.
    utils._BYREF_INT, # Output, m1.
    utils._BYREF_INT, # Output, n1.
    utils._BYREF_INT, # Input/Output, ifail.
)

five_point_summary = utils._EngineState._get_symbol('g01alf')
five_point_summary.restype = None
five_point_summary.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineIntArrayType.c_type, # Workspace, iwrk[n].
    utils._EngineFloat64ArrayType.c_type, # Output, res[5].
    utils._BYREF_INT, # Input/Output, ifail.
)

quantiles = utils._EngineState._get_symbol('g01amf')
quantiles.restype = None
quantiles.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rv[n].
    utils._BYREF_INT, # Input, nq.
    utils._EngineFloat64ArrayType.c_type, # Input, q[nq].
    utils._EngineFloat64ArrayType.c_type, # Output, qv[nq].
    utils._BYREF_INT, # Input/Output, ifail.
)

quantiles_stream_fixed = utils._EngineState._get_symbol('g01anf')
quantiles_stream_fixed.restype = None
quantiles_stream_fixed.argtypes = (
    utils._BYREF_INT, # Input/Output, ind.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, rv[*].
    utils._BYREF_INT, # Input, nb.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._BYREF_INT, # Output, np.
    utils._EngineFloat64ArrayType.c_type, # Input, q[*].
    utils._EngineFloat64ArrayType.c_type, # Output, qv[*].
    utils._BYREF_INT, # Input, nq.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
    utils._BYREF_INT, # Input, licomm.
    utils._BYREF_INT, # Input/Output, ifail.
)

quantiles_stream_arbitrary = utils._EngineState._get_symbol('g01apf')
quantiles_stream_arbitrary.restype = None
quantiles_stream_arbitrary.argtypes = (
    utils._BYREF_INT, # Input/Output, ind.
    utils._EngineFloat64ArrayType.c_type, # Input, rv[*].
    utils._BYREF_INT, # Input, nb.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._BYREF_INT, # Output, np.
    utils._EngineFloat64ArrayType.c_type, # Input, q[*].
    utils._EngineFloat64ArrayType.c_type, # Output, qv[*].
    utils._BYREF_INT, # Input, nq.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
    utils._BYREF_INT, # Input, licomm.
    utils._BYREF_INT, # Input/Output, ifail.
)

plot_stem_leaf = utils._EngineState._get_symbol('g01arf')
plot_stem_leaf.restype = None
plot_stem_leaf.argtypes = (
    utils._BYREF_STRING, # Input, range(len=1).
    utils._BYREF_STRING, # Input, prt(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_INT, # Input, nstepx.
    utils._BYREF_INT, # Input, nstepy.
    utils._BYREF_FLOAT64, # Input/Output, unit.
    utils._EngineCharArrayType.c_type, # Output, plot(len=1)[ldplot, nstepx].
    utils._BYREF_INT, # Input, ldplot.
    utils._BYREF_INT, # Output, lines.
    utils._EngineFloat64ArrayType.c_type, # Output, sorty[n].
    utils._EngineIntArrayType.c_type, # Workspace, iwork[n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of range.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of prt.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of plot.
)

plot_box_whisker = utils._EngineState._get_symbol('g01asf')
plot_box_whisker.restype = None
plot_box_whisker.argtypes = (
    utils._BYREF_STRING, # Input, prt(len=1).
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, n[m].
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, nstepx.
    utils._BYREF_INT, # Input, nstepy.
    utils._EngineCharArrayType.c_type, # Output, plot(len=1)[ldplot, nstepx].
    utils._BYREF_INT, # Input, ldplot.
    utils._EngineFloat64ArrayType.c_type, # Output, fivept[5*m].
    utils._EngineIntArrayType.c_type, # Workspace, iwork[ldx].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of prt.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of plot.
)

summary_onevar = utils._EngineState._get_symbol('g01atf')
summary_onevar.restype = None
summary_onevar.argtypes = (
    utils._BYREF_INT, # Input, nb.
    utils._EngineFloat64ArrayType.c_type, # Input, x[nb].
    utils._BYREF_INT, # Input, iwt.
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_INT, # Input/Output, pn.
    utils._BYREF_FLOAT64, # Output, xmean.
    utils._BYREF_FLOAT64, # Output, xsd.
    utils._BYREF_FLOAT64, # Output, xskew.
    utils._BYREF_FLOAT64, # Output, xkurt.
    utils._BYREF_FLOAT64, # Output, xmin.
    utils._BYREF_FLOAT64, # Output, xmax.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

summary_onevar_combine = utils._EngineState._get_symbol('g01auf')
summary_onevar_combine.restype = None
summary_onevar_combine.argtypes = (
    utils._BYREF_INT, # Input, b.
    utils._EngineFloat64ArrayType.c_type, # Input, mrcomm[20, b].
    utils._BYREF_INT, # Output, pn.
    utils._BYREF_FLOAT64, # Output, xmean.
    utils._BYREF_FLOAT64, # Output, xsd.
    utils._BYREF_FLOAT64, # Output, xskew.
    utils._BYREF_FLOAT64, # Output, xkurt.
    utils._BYREF_FLOAT64, # Output, xmin.
    utils._BYREF_FLOAT64, # Output, xmax.
    utils._EngineFloat64ArrayType.c_type, # Output, rcomm[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_binomial = utils._EngineState._get_symbol('g01bjf')
prob_binomial.restype = None
prob_binomial.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, p.
    utils._BYREF_INT, # Input, k.
    utils._BYREF_FLOAT64, # Output, plek.
    utils._BYREF_FLOAT64, # Output, pgtk.
    utils._BYREF_FLOAT64, # Output, peqk.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_poisson = utils._EngineState._get_symbol('g01bkf')
prob_poisson.restype = None
prob_poisson.argtypes = (
    utils._BYREF_FLOAT64, # Input, rlamda.
    utils._BYREF_INT, # Input, k.
    utils._BYREF_FLOAT64, # Output, plek.
    utils._BYREF_FLOAT64, # Output, pgtk.
    utils._BYREF_FLOAT64, # Output, peqk.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_hypergeom = utils._EngineState._get_symbol('g01blf')
prob_hypergeom.restype = None
prob_hypergeom.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, l.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, k.
    utils._BYREF_FLOAT64, # Output, plek.
    utils._BYREF_FLOAT64, # Output, pgtk.
    utils._BYREF_FLOAT64, # Output, peqk.
    utils._BYREF_INT, # Input/Output, ifail.
)

normal_scores_exact = utils._EngineState._get_symbol('g01daf')
normal_scores_exact.restype = None
normal_scores_exact.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Output, pp[n].
    utils._BYREF_FLOAT64, # Input, etol.
    utils._BYREF_FLOAT64, # Output, errest.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[iw].
    utils._BYREF_INT, # Input, iw.
    utils._BYREF_INT, # Input/Output, ifail.
)

normal_scores_approx = utils._EngineState._get_symbol('g01dbf')
normal_scores_approx.restype = None
normal_scores_approx.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Output, pp[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

normal_scores_var = utils._EngineState._get_symbol('g01dcf')
normal_scores_var.restype = None
normal_scores_var.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, exp1.
    utils._BYREF_FLOAT64, # Input, exp2.
    utils._BYREF_FLOAT64, # Input, sumssq.
    utils._EngineFloat64ArrayType.c_type, # Output, vec[n*(n+1)/2].
    utils._BYREF_INT, # Input/Output, ifail.
)

test_shapiro_wilk = utils._EngineState._get_symbol('g01ddf')
test_shapiro_wilk.restype = None
test_shapiro_wilk.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._BYREF_INT, # Input, n.
    utils._BYREF_LOGICAL, # Input, calwts.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, a[n].
    utils._BYREF_FLOAT64, # Output, w.
    utils._BYREF_FLOAT64, # Output, pw.
    utils._BYREF_INT, # Input/Output, ifail.
)

ranks_and_scores = utils._EngineState._get_symbol('g01dhf')
ranks_and_scores.restype = None
ranks_and_scores.argtypes = (
    utils._BYREF_STRING, # Input, scores(len=1).
    utils._BYREF_STRING, # Input, ties(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, r[n].
    utils._EngineIntArrayType.c_type, # Workspace, iwrk[n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scores.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of ties.
)

prob_normal = utils._EngineState._get_symbol('g01eaf')
prob_normal.restype = utils._EngineFloat64ScalarType.c_type
prob_normal.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_students_t = utils._EngineState._get_symbol('g01ebf')
prob_students_t.restype = utils._EngineFloat64ScalarType.c_type
prob_students_t.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._BYREF_FLOAT64, # Input, t.
    utils._BYREF_FLOAT64, # Input, df.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_chisq = utils._EngineState._get_symbol('g01ecf')
prob_chisq.restype = utils._EngineFloat64ScalarType.c_type
prob_chisq.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, df.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_f = utils._EngineState._get_symbol('g01edf')
prob_f.restype = utils._EngineFloat64ScalarType.c_type
prob_f.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._BYREF_FLOAT64, # Input, f.
    utils._BYREF_FLOAT64, # Input, df1.
    utils._BYREF_FLOAT64, # Input, df2.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_beta = utils._EngineState._get_symbol('g01eef')
prob_beta.restype = None
prob_beta.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_FLOAT64, # Output, p.
    utils._BYREF_FLOAT64, # Output, q.
    utils._BYREF_FLOAT64, # Output, pdf.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_gamma = utils._EngineState._get_symbol('g01eff')
prob_gamma.restype = utils._EngineFloat64ScalarType.c_type
prob_gamma.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._BYREF_FLOAT64, # Input, g.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_studentized_range = utils._EngineState._get_symbol('g01emf')
prob_studentized_range.restype = utils._EngineFloat64ScalarType.c_type
prob_studentized_range.argtypes = (
    utils._BYREF_FLOAT64, # Input, q.
    utils._BYREF_FLOAT64, # Input, v.
    utils._BYREF_INT, # Input, ir.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_durbin_watson = utils._EngineState._get_symbol('g01epf')
prob_durbin_watson.restype = None
prob_durbin_watson.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_FLOAT64, # Input, d.
    utils._BYREF_FLOAT64, # Output, pdl.
    utils._BYREF_FLOAT64, # Output, pdu.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_vonmises = utils._EngineState._get_symbol('g01erf')
prob_vonmises.restype = utils._EngineFloat64ScalarType.c_type
prob_vonmises.argtypes = (
    utils._BYREF_FLOAT64, # Input, t.
    utils._BYREF_FLOAT64, # Input, vk.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_landau = utils._EngineState._get_symbol('g01etf')
prob_landau.restype = utils._EngineFloat64ScalarType.c_type
prob_landau.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
)

prob_vavilov = utils._EngineState._get_symbol('g01euf')
prob_vavilov.restype = utils._EngineFloat64ScalarType.c_type
prob_vavilov.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._EngineFloat64ArrayType.c_type, # Input, rcomm[322].
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_dickey_fuller_unit = utils._EngineState._get_symbol('g01ewf')
prob_dickey_fuller_unit.restype = utils._EngineFloat64ScalarType.c_type
prob_dickey_fuller_unit.argtypes = (
    utils._BYREF_INT, # Input, method.
    utils._BYREF_INT, # Input, type.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, ts.
    utils._BYREF_INT, # Input, nsamp.
    utils._EngineIntArrayType.c_type, # Input/Output, state[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_kolmogorov1 = utils._EngineState._get_symbol('g01eyf')
prob_kolmogorov1.restype = utils._EngineFloat64ScalarType.c_type
prob_kolmogorov1.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, d.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_kolmogorov2 = utils._EngineState._get_symbol('g01ezf')
prob_kolmogorov2.restype = utils._EngineFloat64ScalarType.c_type
prob_kolmogorov2.argtypes = (
    utils._BYREF_INT, # Input, n1.
    utils._BYREF_INT, # Input, n2.
    utils._BYREF_FLOAT64, # Input, d.
    utils._BYREF_INT, # Input/Output, ifail.
)

inv_cdf_normal = utils._EngineState._get_symbol('g01faf')
inv_cdf_normal.restype = utils._EngineFloat64ScalarType.c_type
inv_cdf_normal.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._BYREF_FLOAT64, # Input, p.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

inv_cdf_students_t = utils._EngineState._get_symbol('g01fbf')
inv_cdf_students_t.restype = utils._EngineFloat64ScalarType.c_type
inv_cdf_students_t.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._BYREF_FLOAT64, # Input, p.
    utils._BYREF_FLOAT64, # Input, df.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

inv_cdf_chisq = utils._EngineState._get_symbol('g01fcf')
inv_cdf_chisq.restype = utils._EngineFloat64ScalarType.c_type
inv_cdf_chisq.argtypes = (
    utils._BYREF_FLOAT64, # Input, p.
    utils._BYREF_FLOAT64, # Input, df.
    utils._BYREF_INT, # Input/Output, ifail.
)

inv_cdf_f = utils._EngineState._get_symbol('g01fdf')
inv_cdf_f.restype = utils._EngineFloat64ScalarType.c_type
inv_cdf_f.argtypes = (
    utils._BYREF_FLOAT64, # Input, p.
    utils._BYREF_FLOAT64, # Input, df1.
    utils._BYREF_FLOAT64, # Input, df2.
    utils._BYREF_INT, # Input/Output, ifail.
)

inv_cdf_beta = utils._EngineState._get_symbol('g01fef')
inv_cdf_beta.restype = utils._EngineFloat64ScalarType.c_type
inv_cdf_beta.argtypes = (
    utils._BYREF_FLOAT64, # Input, p.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input/Output, ifail.
)

inv_cdf_gamma = utils._EngineState._get_symbol('g01fff')
inv_cdf_gamma.restype = utils._EngineFloat64ScalarType.c_type
inv_cdf_gamma.argtypes = (
    utils._BYREF_FLOAT64, # Input, p.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input/Output, ifail.
)

inv_cdf_studentized_range = utils._EngineState._get_symbol('g01fmf')
inv_cdf_studentized_range.restype = utils._EngineFloat64ScalarType.c_type
inv_cdf_studentized_range.argtypes = (
    utils._BYREF_FLOAT64, # Input, p.
    utils._BYREF_FLOAT64, # Input, v.
    utils._BYREF_INT, # Input, ir.
    utils._BYREF_INT, # Input/Output, ifail.
)

inv_cdf_landau = utils._EngineState._get_symbol('g01ftf')
inv_cdf_landau.restype = utils._EngineFloat64ScalarType.c_type
inv_cdf_landau.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_students_t_noncentral = utils._EngineState._get_symbol('g01gbf')
prob_students_t_noncentral.restype = utils._EngineFloat64ScalarType.c_type
prob_students_t_noncentral.argtypes = (
    utils._BYREF_FLOAT64, # Input, t.
    utils._BYREF_FLOAT64, # Input, df.
    utils._BYREF_FLOAT64, # Input, delta.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_chisq_noncentral = utils._EngineState._get_symbol('g01gcf')
prob_chisq_noncentral.restype = utils._EngineFloat64ScalarType.c_type
prob_chisq_noncentral.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, df.
    utils._BYREF_FLOAT64, # Input, rlamda.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_f_noncentral = utils._EngineState._get_symbol('g01gdf')
prob_f_noncentral.restype = utils._EngineFloat64ScalarType.c_type
prob_f_noncentral.argtypes = (
    utils._BYREF_FLOAT64, # Input, f.
    utils._BYREF_FLOAT64, # Input, df1.
    utils._BYREF_FLOAT64, # Input, df2.
    utils._BYREF_FLOAT64, # Input, rlamda.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_beta_noncentral = utils._EngineState._get_symbol('g01gef')
prob_beta_noncentral.restype = utils._EngineFloat64ScalarType.c_type
prob_beta_noncentral.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, rlamda.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_bivariate_normal = utils._EngineState._get_symbol('g01haf')
prob_bivariate_normal.restype = utils._EngineFloat64ScalarType.c_type
prob_bivariate_normal.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, y.
    utils._BYREF_FLOAT64, # Input, rho.
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_multi_normal = utils._EngineState._get_symbol('g01hbf')
prob_multi_normal.restype = utils._EngineFloat64ScalarType.c_type
prob_multi_normal.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, a[n].
    utils._EngineFloat64ArrayType.c_type, # Input, b[n].
    utils._EngineFloat64ArrayType.c_type, # Input, xmu[n].
    utils._EngineFloat64ArrayType.c_type, # Input, sig[ldsig, n].
    utils._BYREF_INT, # Input, ldsig.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[lwk].
    utils._BYREF_INT, # Input, lwk.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_bivariate_students_t = utils._EngineState._get_symbol('g01hcf')
prob_bivariate_students_t.restype = utils._EngineFloat64ScalarType.c_type
prob_bivariate_students_t.argtypes = (
    utils._BYREF_STRING, # Input, tail(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input, a[2].
    utils._EngineFloat64ArrayType.c_type, # Input, b[2].
    utils._BYREF_INT, # Input, df.
    utils._BYREF_FLOAT64, # Input, rho.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_multi_students_t = utils._EngineState._get_symbol('g01hdf')
prob_multi_students_t.restype = utils._EngineFloat64ScalarType.c_type
prob_multi_students_t.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[n].
    utils._EngineFloat64ArrayType.c_type, # Input, a[n].
    utils._EngineFloat64ArrayType.c_type, # Input, b[n].
    utils._BYREF_FLOAT64, # Input, nu.
    utils._EngineFloat64ArrayType.c_type, # Input, delta[n].
    utils._BYREF_INT, # Input, iscov.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rc[ldrc, n].
    utils._BYREF_INT, # Input, ldrc.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_INT, # Input, numsub.
    utils._BYREF_INT, # Input, nsampl.
    utils._BYREF_INT, # Input, fmax.
    utils._BYREF_FLOAT64, # Output, errest.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_chisq_noncentral_lincomb = utils._EngineState._get_symbol('g01jcf')
prob_chisq_noncentral_lincomb.restype = None
prob_chisq_noncentral_lincomb.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, a[n].
    utils._EngineIntArrayType.c_type, # Input, mult[n].
    utils._EngineFloat64ArrayType.c_type, # Input, rlamda[n].
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_FLOAT64, # Output, p.
    utils._BYREF_FLOAT64, # Output, pdf.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wrk[n+2*maxit].
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_chisq_lincomb = utils._EngineState._get_symbol('g01jdf')
prob_chisq_lincomb.restype = None
prob_chisq_lincomb.argtypes = (
    utils._BYREF_STRING, # Input, method(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, rlam[n].
    utils._BYREF_FLOAT64, # Input, d.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_FLOAT64, # Output, prob.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[n+1].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of method.
)

pdf_normal = utils._EngineState._get_symbol('g01kaf')
pdf_normal.restype = utils._EngineFloat64ScalarType.c_type
pdf_normal.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, xmean.
    utils._BYREF_FLOAT64, # Input, xstd.
    utils._BYREF_INT, # Input/Output, ifail.
)

pdf_gamma = utils._EngineState._get_symbol('g01kff')
pdf_gamma.restype = utils._EngineFloat64ScalarType.c_type
pdf_gamma.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Input/Output, ifail.
)

pdf_gamma_vector = utils._EngineState._get_symbol('g01kkf')
pdf_gamma_vector.restype = None
pdf_gamma_vector.argtypes = (
    utils._BYREF_INT, # Input, ilog.
    utils._BYREF_INT, # Input, lx.
    utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
    utils._BYREF_INT, # Input, la.
    utils._EngineFloat64ArrayType.c_type, # Input, a[la].
    utils._BYREF_INT, # Input, lb.
    utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, pdf[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

pdf_normal_vector = utils._EngineState._get_symbol('g01kqf')
pdf_normal_vector.restype = None
pdf_normal_vector.argtypes = (
    utils._BYREF_INT, # Input, ilog.
    utils._BYREF_INT, # Input, lx.
    utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
    utils._BYREF_INT, # Input, lxmu.
    utils._EngineFloat64ArrayType.c_type, # Input, xmu[lxmu].
    utils._BYREF_INT, # Input, lxstd.
    utils._EngineFloat64ArrayType.c_type, # Input, xstd[lxstd].
    utils._EngineFloat64ArrayType.c_type, # Output, pdf[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

pdf_multi_normal_vector = utils._EngineState._get_symbol('g01lbf')
pdf_multi_normal_vector.restype = None
pdf_multi_normal_vector.argtypes = (
    utils._BYREF_INT, # Input, ilog.
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, *].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Input, xmu[n].
    utils._BYREF_INT, # Input, iuld.
    utils._EngineFloat64ArrayType.c_type, # Input, sig[ldsig, *].
    utils._BYREF_INT, # Input, ldsig.
    utils._EngineFloat64ArrayType.c_type, # Output, pdf[k].
    utils._BYREF_INT, # Output, rank.
    utils._BYREF_INT, # Input/Output, ifail.
)

mills_ratio = utils._EngineState._get_symbol('g01mbf')
mills_ratio.restype = utils._EngineFloat64ScalarType.c_type
mills_ratio.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
)

pdf_landau = utils._EngineState._get_symbol('g01mtf')
pdf_landau.restype = utils._EngineFloat64ScalarType.c_type
pdf_landau.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
)

pdf_vavilov = utils._EngineState._get_symbol('g01muf')
pdf_vavilov.restype = utils._EngineFloat64ScalarType.c_type
pdf_vavilov.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._EngineFloat64ArrayType.c_type, # Input, rcomm[322].
    utils._BYREF_INT, # Input/Output, ifail.
)

moments_quad_form = utils._EngineState._get_symbol('g01naf')
moments_quad_form.restype = None
moments_quad_form.argtypes = (
    utils._BYREF_STRING, # Input, mom(len=1).
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
    utils._BYREF_INT, # Input, lda.
    utils._EngineFloat64ArrayType.c_type, # Input, emu[*].
    utils._EngineFloat64ArrayType.c_type, # Input, sigma[ldsig, n].
    utils._BYREF_INT, # Input, ldsig.
    utils._BYREF_INT, # Input, l.
    utils._EngineFloat64ArrayType.c_type, # Output, rkum[l].
    utils._EngineFloat64ArrayType.c_type, # Output, rmom[*].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[3*n*(n+1)/2+n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mom.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
)

moments_ratio_quad_forms = utils._EngineState._get_symbol('g01nbf')
moments_ratio_quad_forms.restype = None
moments_ratio_quad_forms.argtypes = (
    utils._BYREF_STRING, # Input, case(len=1).
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
    utils._BYREF_INT, # Input, lda.
    utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, n].
    utils._BYREF_INT, # Input, ldb.
    utils._EngineFloat64ArrayType.c_type, # Input, c[ldc, *].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Input, ela[*].
    utils._EngineFloat64ArrayType.c_type, # Input, emu[*].
    utils._EngineFloat64ArrayType.c_type, # Input, sigma[ldsig, n].
    utils._BYREF_INT, # Input, ldsig.
    utils._BYREF_INT, # Input, l1.
    utils._BYREF_INT, # Input, l2.
    utils._BYREF_INT, # Output, lmax.
    utils._EngineFloat64ArrayType.c_type, # Output, rmom[l2-l1+1].
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[3*n*n+(8+l2)*n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of case.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
)

pdf_landau_moment1 = utils._EngineState._get_symbol('g01ptf')
pdf_landau_moment1.restype = utils._EngineFloat64ScalarType.c_type
pdf_landau_moment1.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
)

pdf_landau_moment2 = utils._EngineState._get_symbol('g01qtf')
pdf_landau_moment2.restype = utils._EngineFloat64ScalarType.c_type
pdf_landau_moment2.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
)

pdf_landau_deriv = utils._EngineState._get_symbol('g01rtf')
pdf_landau_deriv.restype = utils._EngineFloat64ScalarType.c_type
pdf_landau_deriv.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
)

prob_normal_vector = utils._EngineState._get_symbol('g01saf')
prob_normal_vector.restype = None
prob_normal_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lx.
    utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
    utils._BYREF_INT, # Input, lxmu.
    utils._EngineFloat64ArrayType.c_type, # Input, xmu[lxmu].
    utils._BYREF_INT, # Input, lxstd.
    utils._EngineFloat64ArrayType.c_type, # Input, xstd[lxstd].
    utils._EngineFloat64ArrayType.c_type, # Output, p[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_students_t_vector = utils._EngineState._get_symbol('g01sbf')
prob_students_t_vector.restype = None
prob_students_t_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lt.
    utils._EngineFloat64ArrayType.c_type, # Input, t[lt].
    utils._BYREF_INT, # Input, ldf.
    utils._EngineFloat64ArrayType.c_type, # Input, df[ldf].
    utils._EngineFloat64ArrayType.c_type, # Output, p[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_chisq_vector = utils._EngineState._get_symbol('g01scf')
prob_chisq_vector.restype = None
prob_chisq_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lx.
    utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
    utils._BYREF_INT, # Input, ldf.
    utils._EngineFloat64ArrayType.c_type, # Input, df[ldf].
    utils._EngineFloat64ArrayType.c_type, # Output, p[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_f_vector = utils._EngineState._get_symbol('g01sdf')
prob_f_vector.restype = None
prob_f_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lf.
    utils._EngineFloat64ArrayType.c_type, # Input, f[lf].
    utils._BYREF_INT, # Input, ldf1.
    utils._EngineFloat64ArrayType.c_type, # Input, df1[ldf1].
    utils._BYREF_INT, # Input, ldf2.
    utils._EngineFloat64ArrayType.c_type, # Input, df2[ldf2].
    utils._EngineFloat64ArrayType.c_type, # Output, p[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_beta_vector = utils._EngineState._get_symbol('g01sef')
prob_beta_vector.restype = None
prob_beta_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lbeta.
    utils._EngineFloat64ArrayType.c_type, # Input, beta[lbeta].
    utils._BYREF_INT, # Input, la.
    utils._EngineFloat64ArrayType.c_type, # Input, a[la].
    utils._BYREF_INT, # Input, lb.
    utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, p[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_gamma_vector = utils._EngineState._get_symbol('g01sff')
prob_gamma_vector.restype = None
prob_gamma_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lg.
    utils._EngineFloat64ArrayType.c_type, # Input, g[lg].
    utils._BYREF_INT, # Input, la.
    utils._EngineFloat64ArrayType.c_type, # Input, a[la].
    utils._BYREF_INT, # Input, lb.
    utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, p[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

prob_binomial_vector = utils._EngineState._get_symbol('g01sjf')
prob_binomial_vector.restype = None
prob_binomial_vector.argtypes = (
    utils._BYREF_INT, # Input, ln.
    utils._EngineIntArrayType.c_type, # Input, n[ln].
    utils._BYREF_INT, # Input, lp.
    utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
    utils._BYREF_INT, # Input, lk.
    utils._EngineIntArrayType.c_type, # Input, k[lk].
    utils._EngineFloat64ArrayType.c_type, # Output, plek[*].
    utils._EngineFloat64ArrayType.c_type, # Output, pgtk[*].
    utils._EngineFloat64ArrayType.c_type, # Output, peqk[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_poisson_vector = utils._EngineState._get_symbol('g01skf')
prob_poisson_vector.restype = None
prob_poisson_vector.argtypes = (
    utils._BYREF_INT, # Input, ll.
    utils._EngineFloat64ArrayType.c_type, # Input, l[ll].
    utils._BYREF_INT, # Input, lk.
    utils._EngineIntArrayType.c_type, # Input, k[lk].
    utils._EngineFloat64ArrayType.c_type, # Output, plek[*].
    utils._EngineFloat64ArrayType.c_type, # Output, pgtk[*].
    utils._EngineFloat64ArrayType.c_type, # Output, peqk[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

prob_hypergeom_vector = utils._EngineState._get_symbol('g01slf')
prob_hypergeom_vector.restype = None
prob_hypergeom_vector.argtypes = (
    utils._BYREF_INT, # Input, ln.
    utils._EngineIntArrayType.c_type, # Input, n[ln].
    utils._BYREF_INT, # Input, ll.
    utils._EngineIntArrayType.c_type, # Input, l[ll].
    utils._BYREF_INT, # Input, lm.
    utils._EngineIntArrayType.c_type, # Input, m[lm].
    utils._BYREF_INT, # Input, lk.
    utils._EngineIntArrayType.c_type, # Input, k[lk].
    utils._EngineFloat64ArrayType.c_type, # Output, plek[*].
    utils._EngineFloat64ArrayType.c_type, # Output, pgtk[*].
    utils._EngineFloat64ArrayType.c_type, # Output, peqk[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

inv_cdf_normal_vector = utils._EngineState._get_symbol('g01taf')
inv_cdf_normal_vector.restype = None
inv_cdf_normal_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lp.
    utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
    utils._BYREF_INT, # Input, lxmu.
    utils._EngineFloat64ArrayType.c_type, # Input, xmu[lxmu].
    utils._BYREF_INT, # Input, lxstd.
    utils._EngineFloat64ArrayType.c_type, # Input, xstd[lxstd].
    utils._EngineFloat64ArrayType.c_type, # Output, x[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

inv_cdf_students_t_vector = utils._EngineState._get_symbol('g01tbf')
inv_cdf_students_t_vector.restype = None
inv_cdf_students_t_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lp.
    utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
    utils._BYREF_INT, # Input, ldf.
    utils._EngineFloat64ArrayType.c_type, # Input, df[ldf].
    utils._EngineFloat64ArrayType.c_type, # Output, t[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

inv_cdf_chisq_vector = utils._EngineState._get_symbol('g01tcf')
inv_cdf_chisq_vector.restype = None
inv_cdf_chisq_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lp.
    utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
    utils._BYREF_INT, # Input, ldf.
    utils._EngineFloat64ArrayType.c_type, # Input, df[ldf].
    utils._EngineFloat64ArrayType.c_type, # Output, x[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

inv_cdf_f_vector = utils._EngineState._get_symbol('g01tdf')
inv_cdf_f_vector.restype = None
inv_cdf_f_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lp.
    utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
    utils._BYREF_INT, # Input, ldf1.
    utils._EngineFloat64ArrayType.c_type, # Input, df1[ldf1].
    utils._BYREF_INT, # Input, ldf2.
    utils._EngineFloat64ArrayType.c_type, # Input, df2[ldf2].
    utils._EngineFloat64ArrayType.c_type, # Output, f[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

inv_cdf_beta_vector = utils._EngineState._get_symbol('g01tef')
inv_cdf_beta_vector.restype = None
inv_cdf_beta_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lp.
    utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
    utils._BYREF_INT, # Input, la.
    utils._EngineFloat64ArrayType.c_type, # Input, a[la].
    utils._BYREF_INT, # Input, lb.
    utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Output, beta[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

inv_cdf_gamma_vector = utils._EngineState._get_symbol('g01tff')
inv_cdf_gamma_vector.restype = None
inv_cdf_gamma_vector.argtypes = (
    utils._BYREF_INT, # Input, ltail.
    utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
    utils._BYREF_INT, # Input, lp.
    utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
    utils._BYREF_INT, # Input, la.
    utils._EngineFloat64ArrayType.c_type, # Input, a[la].
    utils._BYREF_INT, # Input, lb.
    utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Output, g[*].
    utils._EngineIntArrayType.c_type, # Output, ivalid[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
)

moving_average = utils._EngineState._get_symbol('g01waf')
moving_average.restype = None
moving_average.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, nb.
    utils._EngineFloat64ArrayType.c_type, # Input, x[nb].
    utils._BYREF_INT, # Input, iwt.
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_INT, # Input/Output, pn.
    utils._EngineFloat64ArrayType.c_type, # Output, rmean[max(0, nb+min(0, pn-m+1))].
    utils._EngineFloat64ArrayType.c_type, # Output, rsd[lrsd].
    utils._BYREF_INT, # Input, lrsd.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._BYREF_INT, # Input/Output, ifail.
)

init_vavilov = utils._EngineState._get_symbol('g01zuf')
init_vavilov.restype = None
init_vavilov.argtypes = (
    utils._BYREF_FLOAT64, # Input, rkappa.
    utils._BYREF_FLOAT64, # Input, beta2.
    utils._BYREF_INT, # Input, mode.
    utils._BYREF_FLOAT64, # Output, xl.
    utils._BYREF_FLOAT64, # Output, xu.
    utils._EngineFloat64ArrayType.c_type, # Output, rcomm[322].
    utils._BYREF_INT, # Input/Output, ifail.
)
