# -*- coding: utf-8 -*-
# pylint: disable=invalid-name,line-too-long,protected-access,too-many-locals
# pylint: disable=too-many-lines
"""
Primitive ``ctypes`` prototypes for the NAG Library `sum` Chapter.
"""

# NAG Copyright 2017-2019.

from ..base import utils

accelerate = utils._EngineState._get_symbol('c06baf')
accelerate.restype = None
accelerate.argtypes = (
    utils._BYREF_FLOAT64, # Input, seqn.
    utils._BYREF_INT, # Input/Output, ncall.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
)

chebyshev = utils._EngineState._get_symbol('c06dcf')
chebyshev.restype = None
chebyshev.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
    utils._BYREF_INT, # Input, lx.
    utils._BYREF_FLOAT64, # Input, xmin.
    utils._BYREF_FLOAT64, # Input, xmax.
    utils._EngineFloat64ArrayType.c_type, # Input, c[n].
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Output, res[lx].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_real_1d_rfmt = utils._EngineState._get_symbol('c06faf')
fft_real_1d_rfmt.restype = None
fft_real_1d_rfmt.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_hermitian_1d_rfmt = utils._EngineState._get_symbol('c06fbf')
fft_hermitian_1d_rfmt.restype = None
fft_hermitian_1d_rfmt.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_complex_1d_sep = utils._EngineState._get_symbol('c06fcf')
fft_complex_1d_sep.restype = None
fft_complex_1d_sep.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_complex_multid_1d_sep = utils._EngineState._get_symbol('c06fff')
fft_complex_multid_1d_sep.restype = None
fft_complex_multid_1d_sep.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._BYREF_INT, # Input, l.
    utils._EngineIntArrayType.c_type, # Input, nd[ndim].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_complex_multid_sep = utils._EngineState._get_symbol('c06fjf')
fft_complex_multid_sep.restype = None
fft_complex_multid_sep.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._EngineIntArrayType.c_type, # Input, nd[ndim].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
)

convcorr_real = utils._EngineState._get_symbol('c06fkf')
convcorr_real.restype = None
convcorr_real.argtypes = (
    utils._BYREF_INT, # Input, job.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

withdraw_fft_real_1d_multi_rfmt = utils._EngineState._get_symbol('c06fpf')
withdraw_fft_real_1d_multi_rfmt.restype = None
withdraw_fft_real_1d_multi_rfmt.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[m*n].
    utils._BYREF_STRING, # Input, init(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input/Output, trig[2*n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[m*n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of init.
)

withdraw_fft_hermitian_1d_multi_rfmt = utils._EngineState._get_symbol('c06fqf')
withdraw_fft_hermitian_1d_multi_rfmt.restype = None
withdraw_fft_hermitian_1d_multi_rfmt.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[m*n].
    utils._BYREF_STRING, # Input, init(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input/Output, trig[2*n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[m*n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of init.
)

fft_complex_3d_sep = utils._EngineState._get_symbol('c06fxf')
fft_complex_3d_sep.restype = None
fft_complex_3d_sep.argtypes = (
    utils._BYREF_INT, # Input, n1.
    utils._BYREF_INT, # Input, n2.
    utils._BYREF_INT, # Input, n3.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n1*n2*n3].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n1*n2*n3].
    utils._BYREF_STRING, # Input, init(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input/Output, trign1[1].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, trign2[1].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, trign3[1].
    utils._EngineFloat64ArrayType.c_type, # Output, work[1].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of init.
)

invlaplace_crump_f = utils._EngineState._sys_functype(
    None,
    utils._BYREF_FLOAT64, # Input, pr.
    utils._BYREF_FLOAT64, # Input, pi.
    utils._BYREF_FLOAT64, # Output, fr.
    utils._BYREF_FLOAT64, # Output, fi.
)
invlaplace_crump = utils._EngineState._get_symbol('c06laf')
invlaplace_crump.restype = None
invlaplace_crump.argtypes = (
    invlaplace_crump_f,
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._EngineFloat64ArrayType.c_type, # Output, valinv[n].
    utils._EngineFloat64ArrayType.c_type, # Output, errest[n].
    utils._BYREF_FLOAT64, # Input, relerr.
    utils._BYREF_FLOAT64, # Input, alphab.
    utils._BYREF_FLOAT64, # Input, tfac.
    utils._BYREF_INT, # Input, mxterm.
    utils._BYREF_INT, # Output, nterms.
    utils._BYREF_INT, # Output, na.
    utils._BYREF_FLOAT64, # Output, alow.
    utils._BYREF_FLOAT64, # Output, ahigh.
    utils._BYREF_INT, # Output, nfeval.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[4*mxterm+2].
    utils._BYREF_INT, # Input/Output, ifail.
)

invlaplace_weeks = None # Not supported: com-valued callback, f

invlaplace_weeks_eval = utils._EngineState._get_symbol('c06lcf')
invlaplace_weeks_eval.restype = None
invlaplace_weeks_eval.argtypes = (
    utils._BYREF_FLOAT64, # Input, t.
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, acoef[m].
    utils._EngineFloat64ArrayType.c_type, # Input, errvec[8].
    utils._BYREF_FLOAT64, # Output, finv.
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_realherm_1d = utils._EngineState._get_symbol('c06paf')
fft_realherm_1d.restype = None
fft_realherm_1d.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n+2].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_complex_1d = utils._EngineState._get_symbol('c06pcf')
fft_complex_1d.restype = None
fft_complex_1d.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._EngineComplex128ArrayType.c_type, # Input/Output, x[n].
    utils._BYREF_INT, # Input, n.
    utils._EngineComplex128ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_complex_multid_1d = utils._EngineState._get_symbol('c06pff')
fft_complex_multid_1d.restype = None
fft_complex_multid_1d.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, ndim.
    utils._BYREF_INT, # Input, l.
    utils._EngineIntArrayType.c_type, # Input, nd[ndim].
    utils._BYREF_INT, # Input, n.
    utils._EngineComplex128ArrayType.c_type, # Input/Output, x[n].
    utils._EngineComplex128ArrayType.c_type, # Output, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_complex_multid = utils._EngineState._get_symbol('c06pjf')
fft_complex_multid.restype = None
fft_complex_multid.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, ndim.
    utils._EngineIntArrayType.c_type, # Input, nd[ndim].
    utils._BYREF_INT, # Input, n.
    utils._EngineComplex128ArrayType.c_type, # Input/Output, x[n].
    utils._EngineComplex128ArrayType.c_type, # Output, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

convcorr_complex = utils._EngineState._get_symbol('c06pkf')
convcorr_complex.restype = None
convcorr_complex.argtypes = (
    utils._BYREF_INT, # Input, job.
    utils._EngineComplex128ArrayType.c_type, # Input/Output, x[n].
    utils._EngineComplex128ArrayType.c_type, # Input/Output, y[n].
    utils._BYREF_INT, # Input, n.
    utils._EngineComplex128ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_realherm_1d_multi_row = utils._EngineState._get_symbol('c06ppf')
fft_realherm_1d_multi_row.restype = None
fft_realherm_1d_multi_row.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[m*(n+2)].
    utils._EngineFloat64ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_realherm_1d_multi_col = utils._EngineState._get_symbol('c06pqf')
fft_realherm_1d_multi_col.restype = None
fft_realherm_1d_multi_col.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[(n+2)*m].
    utils._EngineFloat64ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_complex_1d_multi_row = utils._EngineState._get_symbol('c06prf')
fft_complex_1d_multi_row.restype = None
fft_complex_1d_multi_row.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineComplex128ArrayType.c_type, # Input/Output, x[m*n].
    utils._EngineComplex128ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_complex_1d_multi_col = utils._EngineState._get_symbol('c06psf')
fft_complex_1d_multi_col.restype = None
fft_complex_1d_multi_col.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineComplex128ArrayType.c_type, # Input/Output, x[n*m].
    utils._EngineComplex128ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_complex_2d = utils._EngineState._get_symbol('c06puf')
fft_complex_2d.restype = None
fft_complex_2d.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineComplex128ArrayType.c_type, # Input/Output, x[m*n].
    utils._EngineComplex128ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_real_2d = utils._EngineState._get_symbol('c06pvf')
fft_real_2d.restype = None
fft_real_2d.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m*n].
    utils._EngineComplex128ArrayType.c_type, # Output, y[(m/2+1)*n].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_hermitian_2d = utils._EngineState._get_symbol('c06pwf')
fft_hermitian_2d.restype = None
fft_hermitian_2d.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineComplex128ArrayType.c_type, # Input, y[(m/2+1)*n].
    utils._EngineFloat64ArrayType.c_type, # Output, x[m*n].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_complex_3d = utils._EngineState._get_symbol('c06pxf')
fft_complex_3d.restype = None
fft_complex_3d.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, n1.
    utils._BYREF_INT, # Input, n2.
    utils._BYREF_INT, # Input, n3.
    utils._EngineComplex128ArrayType.c_type, # Input/Output, x[n1*n2*n3].
    utils._EngineComplex128ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_real_3d = utils._EngineState._get_symbol('c06pyf')
fft_real_3d.restype = None
fft_real_3d.argtypes = (
    utils._BYREF_INT, # Input, n1.
    utils._BYREF_INT, # Input, n2.
    utils._BYREF_INT, # Input, n3.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n1*n2*n3].
    utils._EngineComplex128ArrayType.c_type, # Output, y[(n1/2+1)*n2*n3].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_hermitian_3d = utils._EngineState._get_symbol('c06pzf')
fft_hermitian_3d.restype = None
fft_hermitian_3d.argtypes = (
    utils._BYREF_INT, # Input, n1.
    utils._BYREF_INT, # Input, n2.
    utils._BYREF_INT, # Input, n3.
    utils._EngineComplex128ArrayType.c_type, # Input, y[(n1/2+1)*n2*n3].
    utils._EngineFloat64ArrayType.c_type, # Output, x[n1*n2*n3].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_real_sine_simple = utils._EngineState._get_symbol('c06raf')
fft_real_sine_simple.restype = None
fft_real_sine_simple.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[m*(n+2)].
    utils._EngineFloat64ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_real_cosine_simple = utils._EngineState._get_symbol('c06rbf')
fft_real_cosine_simple.restype = None
fft_real_cosine_simple.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[m*(n+3)].
    utils._EngineFloat64ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_real_qtrsine_simple = utils._EngineState._get_symbol('c06rcf')
fft_real_qtrsine_simple.restype = None
fft_real_qtrsine_simple.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[m*(n+2)].
    utils._EngineFloat64ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_real_qtrcosine_simple = utils._EngineState._get_symbol('c06rdf')
fft_real_qtrcosine_simple.restype = None
fft_real_qtrcosine_simple.argtypes = (
    utils._BYREF_STRING, # Input, direct(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[m*(n+2)].
    utils._EngineFloat64ArrayType.c_type, # Output, work[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of direct.
)

fft_sine = utils._EngineState._get_symbol('c06ref')
fft_sine.restype = None
fft_sine.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n-1, m].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_cosine = utils._EngineState._get_symbol('c06rff')
fft_cosine.restype = None
fft_cosine.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n+1, m].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_qtrsine = utils._EngineState._get_symbol('c06rgf')
fft_qtrsine.restype = None
fft_qtrsine.argtypes = (
    utils._BYREF_INT, # Input, idir.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n, m].
    utils._BYREF_INT, # Input/Output, ifail.
)

fft_qtrcosine = utils._EngineState._get_symbol('c06rhf')
fft_qtrcosine.restype = None
fft_qtrcosine.argtypes = (
    utils._BYREF_INT, # Input, idir.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n, m].
    utils._BYREF_INT, # Input/Output, ifail.
)

fast_gauss = utils._EngineState._get_symbol('c06saf')
fast_gauss.restype = None
fast_gauss.argtypes = (
    utils._BYREF_INT, # Input, d.
    utils._EngineFloat64ArrayType.c_type, # Input, srcs[d, n].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, trgs[d, m].
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, q[n].
    utils._BYREF_INT, # Input/Output, p1.
    utils._BYREF_INT, # Input/Output, p2.
    utils._BYREF_INT, # Input/Output, k.
    utils._EngineFloat64ArrayType.c_type, # Input, hin[lhin].
    utils._BYREF_INT, # Input, lhin.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Output, v[m].
    utils._EngineFloat64ArrayType.c_type, # Output, term[m].
    utils._BYREF_INT, # Input/Output, ifail.
)
