# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `math` Chapter.

``math`` - Mathematical Constants

This module is concerned with the provision of mathematical constants
required by other functions within the NAG Library.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x01/x01intro.html
"""

# NAG Copyright 2017-2019.

def pi():
    u"""
    Provides the mathematical constant pi.

    ``pi`` returns the numerical value of pi.

    For full information please refer to the NAG Library document for
    x01aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x01/x01aaf.html

    Returns
    -------
    pi_nag : float
        The numerical value of pi.
    """

    from ..base.math import pi as b_pi

    _return_scalar = b_pi(
    )
    return _return_scalar

def euler():
    u"""
    Provides the mathematical constant gamma (Euler's constant).

    ``euler`` returns the numerical value of gamma (Euler's constant).

    For full information please refer to the NAG Library document for
    x01ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x01/x01abf.html

    Returns
    -------
    euler : float
        The numerical value of gamma (Euler's constant).
    """

    from ..base.math import euler as b_euler

    _return_scalar = b_euler(
    )
    return _return_scalar
