"""
Deprecated wrappers for the NAG C Library (Mark 26.0) a02 Chapter.

a02 - Complex Arithmetic.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/A02/a02conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from .util import get_library
from .util import Nag_Boolean
from .util import Complex
from .util import Integer
from .util import Nag_Complex_Type

_library_cl = get_library()

a02bac_ctypes = _library_cl.a02bac
a02bac_ctypes.restype = Complex
a02bac_ctypes.argtypes = [c_double, c_double]


def a02bac(x, y):
    """
    Complex number from real and imaginary parts.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02bac.html
    """
    return_value = a02bac_ctypes(x, y)
    return return_value

a02bbc_ctypes = _library_cl.a02bbc
a02bbc_ctypes.restype = c_double
a02bbc_ctypes.argtypes = [Nag_Complex_Type]


def a02bbc(z):
    """
    Real part of a complex number.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02bbc.html
    """
    return_value = a02bbc_ctypes(z)
    return return_value

a02bcc_ctypes = _library_cl.a02bcc
a02bcc_ctypes.restype = c_double
a02bcc_ctypes.argtypes = [Nag_Complex_Type]


def a02bcc(z):
    """
    Imaginary part of a complex number.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02bcc.html
    """
    return_value = a02bcc_ctypes(z)
    return return_value

a02cac_ctypes = _library_cl.a02cac
a02cac_ctypes.restype = Complex
a02cac_ctypes.argtypes = [Nag_Complex_Type, Nag_Complex_Type]


def a02cac(z1, z2):
    """
    Addition of two complex numbers.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02cac.html
    """
    return_value = a02cac_ctypes(z1, z2)
    return return_value

a02cbc_ctypes = _library_cl.a02cbc
a02cbc_ctypes.restype = Complex
a02cbc_ctypes.argtypes = [Nag_Complex_Type, Nag_Complex_Type]


def a02cbc(z1, z2):
    """
    Subtraction of two complex numbers.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02cbc.html
    """
    return_value = a02cbc_ctypes(z1, z2)
    return return_value

a02ccc_ctypes = _library_cl.a02ccc
a02ccc_ctypes.restype = Complex
a02ccc_ctypes.argtypes = [Nag_Complex_Type, Nag_Complex_Type]


def a02ccc(z1, z2):
    """
    Multiplication of two complex numbers.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02ccc.html
    """
    return_value = a02ccc_ctypes(z1, z2)
    return return_value

a02cdc_ctypes = _library_cl.a02cdc
a02cdc_ctypes.restype = Complex
a02cdc_ctypes.argtypes = [Nag_Complex_Type, Nag_Complex_Type]


def a02cdc(z1, z2):
    """
    Quotient of two complex numbers.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02cdc.html
    """
    return_value = a02cdc_ctypes(z1, z2)
    return return_value

a02cec_ctypes = _library_cl.a02cec
a02cec_ctypes.restype = Complex
a02cec_ctypes.argtypes = [Nag_Complex_Type]


def a02cec(z):
    """
    Negation of a complex number.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02cec.html
    """
    return_value = a02cec_ctypes(z)
    return return_value

a02cfc_ctypes = _library_cl.a02cfc
a02cfc_ctypes.restype = Complex
a02cfc_ctypes.argtypes = [Nag_Complex_Type]


def a02cfc(z):
    """
    Conjugate of a complex number.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02cfc.html
    """
    return_value = a02cfc_ctypes(z)
    return return_value

a02cgc_ctypes = _library_cl.a02cgc
a02cgc_ctypes.restype = Nag_Boolean
a02cgc_ctypes.argtypes = [Nag_Complex_Type, Nag_Complex_Type]


def a02cgc(z1, z2):
    """
    Equality of two complex numbers.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02cgc.html
    """
    return_value = a02cgc_ctypes(z1, z2)
    return return_value

a02chc_ctypes = _library_cl.a02chc
a02chc_ctypes.restype = Nag_Boolean
a02chc_ctypes.argtypes = [Nag_Complex_Type, Nag_Complex_Type]


def a02chc(z1, z2):
    """
    Inequality of two complex numbers.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02chc.html
    """
    return_value = a02chc_ctypes(z1, z2)
    return return_value

a02dac_ctypes = _library_cl.a02dac
a02dac_ctypes.restype = c_double
a02dac_ctypes.argtypes = [Nag_Complex_Type]


def a02dac(z):
    """
    Argument of a complex number.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dac.html
    """
    return_value = a02dac_ctypes(z)
    return return_value

a02dbc_ctypes = _library_cl.a02dbc
a02dbc_ctypes.restype = c_double
a02dbc_ctypes.argtypes = [Nag_Complex_Type]


def a02dbc(z):
    """
    Modulus of a complex number.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dbc.html
    """
    return_value = a02dbc_ctypes(z)
    return return_value

a02dcc_ctypes = _library_cl.a02dcc
a02dcc_ctypes.restype = Complex
a02dcc_ctypes.argtypes = [Nag_Complex_Type]


def a02dcc(z):
    """
    Square root of a complex number.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dcc.html
    """
    return_value = a02dcc_ctypes(z)
    return return_value

a02ddc_ctypes = _library_cl.a02ddc
a02ddc_ctypes.restype = Complex
a02ddc_ctypes.argtypes = [Nag_Complex_Type, Integer]


def a02ddc(z, i):
    """
    Complex number raised to integer power.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02ddc.html
    """
    return_value = a02ddc_ctypes(z, i)
    return return_value

a02dec_ctypes = _library_cl.a02dec
a02dec_ctypes.restype = Complex
a02dec_ctypes.argtypes = [Nag_Complex_Type, c_double]


def a02dec(z, x):
    """
    Complex number raised to real power.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dec.html
    """
    return_value = a02dec_ctypes(z, x)
    return return_value

a02dfc_ctypes = _library_cl.a02dfc
a02dfc_ctypes.restype = Complex
a02dfc_ctypes.argtypes = [Nag_Complex_Type, Nag_Complex_Type]


def a02dfc(z1, z2):
    """
    Complex number raised to complex power.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dfc.html
    """
    return_value = a02dfc_ctypes(z1, z2)
    return return_value

a02dgc_ctypes = _library_cl.a02dgc
a02dgc_ctypes.restype = Complex
a02dgc_ctypes.argtypes = [Nag_Complex_Type]


def a02dgc(z):
    """
    Complex logarithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dgc.html
    """
    return_value = a02dgc_ctypes(z)
    return return_value

a02dhc_ctypes = _library_cl.a02dhc
a02dhc_ctypes.restype = Complex
a02dhc_ctypes.argtypes = [Nag_Complex_Type]


def a02dhc(z):
    """
    Complex exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dhc.html
    """
    return_value = a02dhc_ctypes(z)
    return return_value

a02djc_ctypes = _library_cl.a02djc
a02djc_ctypes.restype = Complex
a02djc_ctypes.argtypes = [Nag_Complex_Type]


def a02djc(z):
    """
    Complex sine.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02djc.html
    """
    return_value = a02djc_ctypes(z)
    return return_value

a02dkc_ctypes = _library_cl.a02dkc
a02dkc_ctypes.restype = Complex
a02dkc_ctypes.argtypes = [Nag_Complex_Type]


def a02dkc(z):
    """
    Complex cosine.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dkc.html
    """
    return_value = a02dkc_ctypes(z)
    return return_value

a02dlc_ctypes = _library_cl.a02dlc
a02dlc_ctypes.restype = Complex
a02dlc_ctypes.argtypes = [Nag_Complex_Type]


def a02dlc(z):
    """
    Complex tangent.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/a02/a02dlc.html
    """
    return_value = a02dlc_ctypes(z)
    return return_value

nag_complex = a02bac
nag_complex_real = a02bbc
nag_complex_imag = a02bcc
nag_complex_add = a02cac
nag_complex_subtract = a02cbc
nag_complex_multiply = a02ccc
nag_complex_divide = a02cdc
nag_complex_negate = a02cec
nag_complex_conjg = a02cfc
nag_complex_equal = a02cgc
nag_complex_not_equal = a02chc
nag_complex_arg = a02dac
nag_complex_abs = a02dbc
nag_complex_sqrt = a02dcc
nag_complex_i_power = a02ddc
nag_complex_r_power = a02dec
nag_complex_c_power = a02dfc
nag_complex_log = a02dgc
nag_complex_exp = a02dhc
nag_complex_sin = a02djc
nag_complex_cos = a02dkc
nag_complex_tan = a02dlc
__all__ = ['a02bac', 'a02bbc', 'a02bcc', 'a02cac', 'a02cbc', 'a02ccc',
           'a02cdc', 'a02cec', 'a02cfc', 'a02cgc', 'a02chc', 'a02dac',
           'a02dbc', 'a02dcc', 'a02ddc', 'a02dec', 'a02dfc', 'a02dgc',
           'a02dhc', 'a02djc', 'a02dkc', 'a02dlc', 'nag_complex',
           'nag_complex_abs', 'nag_complex_add', 'nag_complex_arg',
           'nag_complex_c_power', 'nag_complex_conjg',
           'nag_complex_cos', 'nag_complex_divide',
           'nag_complex_equal', 'nag_complex_exp',
           'nag_complex_i_power', 'nag_complex_imag',
           'nag_complex_log', 'nag_complex_multiply',
           'nag_complex_negate', 'nag_complex_not_equal',
           'nag_complex_r_power', 'nag_complex_real',
           'nag_complex_sin', 'nag_complex_sqrt',
           'nag_complex_subtract', 'nag_complex_tan']
