"""
Deprecated wrappers for the NAG C Library (Mark 26.0) s Chapter.

s - Approximations of Special Functions.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/S/sconts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER
from .util import get_library

from .util import NagError
from .util import Integer
from .util import Complex
from .util import Nag_ScaleResType
from .util import Nag_FunType
from .util import Nag_OrderType
from .util import Nag_CallPut
from .util import Nag_Barrier
from .util import Nag_Complex_Type

_library_cl = get_library()

s01bac_ctypes = _library_cl.s01bac
s01bac_ctypes.restype = c_double
s01bac_ctypes.argtypes = [c_double, POINTER(NagError)]


def s01bac(x, fail):
    """
    ln(1+x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s01bac.html
    """
    return_value = s01bac_ctypes(x, fail)
    return return_value

s10aac_ctypes = _library_cl.s10aac
s10aac_ctypes.restype = c_double
s10aac_ctypes.argtypes = [c_double]


def s10aac(x):
    """
    Hyperbolic tangent, tanh x.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s10aac.html
    """
    return_value = s10aac_ctypes(x)
    return return_value

s10abc_ctypes = _library_cl.s10abc
s10abc_ctypes.restype = c_double
s10abc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s10abc(x, fail):
    """
    Hyperbolic sine, sinh x.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s10abc.html
    """
    return_value = s10abc_ctypes(x, fail)
    return return_value

s10acc_ctypes = _library_cl.s10acc
s10acc_ctypes.restype = c_double
s10acc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s10acc(x, fail):
    """
    Hyperbolic cosine, cosh x.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s10acc.html
    """
    return_value = s10acc_ctypes(x, fail)
    return return_value

s11aac_ctypes = _library_cl.s11aac
s11aac_ctypes.restype = c_double
s11aac_ctypes.argtypes = [c_double, POINTER(NagError)]


def s11aac(x, fail):
    """
    Inverse hyperbolic tangent, arctanh x.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s11aac.html
    """
    return_value = s11aac_ctypes(x, fail)
    return return_value

s11abc_ctypes = _library_cl.s11abc
s11abc_ctypes.restype = c_double
s11abc_ctypes.argtypes = [c_double]


def s11abc(x):
    """
    Inverse hyperbolic sine, arcsinh x.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s11abc.html
    """
    return_value = s11abc_ctypes(x)
    return return_value

s11acc_ctypes = _library_cl.s11acc
s11acc_ctypes.restype = c_double
s11acc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s11acc(x, fail):
    """
    Inverse hyperbolic cosine, arccosh x.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s11acc.html
    """
    return_value = s11acc_ctypes(x, fail)
    return return_value

s13aac_ctypes = _library_cl.s13aac
s13aac_ctypes.restype = c_double
s13aac_ctypes.argtypes = [c_double, POINTER(NagError)]


def s13aac(x, fail):
    """
    Exponential integral E_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s13aac.html
    """
    return_value = s13aac_ctypes(x, fail)
    return return_value

s13acc_ctypes = _library_cl.s13acc
s13acc_ctypes.restype = c_double
s13acc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s13acc(x, fail):
    """
    Cosine integral Ci(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s13acc.html
    """
    return_value = s13acc_ctypes(x, fail)
    return return_value

s13adc_ctypes = _library_cl.s13adc
s13adc_ctypes.restype = c_double
s13adc_ctypes.argtypes = [c_double]


def s13adc(x):
    """
    Sine integral Si(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s13adc.html
    """
    return_value = s13adc_ctypes(x)
    return return_value

s14aac_ctypes = _library_cl.s14aac
s14aac_ctypes.restype = c_double
s14aac_ctypes.argtypes = [c_double, POINTER(NagError)]


def s14aac(x, fail):
    """
    Gamma function Gamma(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14aac.html
    """
    return_value = s14aac_ctypes(x, fail)
    return return_value

s14abc_ctypes = _library_cl.s14abc
s14abc_ctypes.restype = c_double
s14abc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s14abc(x, fail):
    """
    Log gamma function ln(Gamma(x)).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14abc.html
    """
    return_value = s14abc_ctypes(x, fail)
    return return_value

s14acc_ctypes = _library_cl.s14acc
s14acc_ctypes.restype = c_double
s14acc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s14acc(x, fail):
    """
    *(x)-ln(x) where *(x) is the psi function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14acc.html
    """
    return_value = s14acc_ctypes(x, fail)
    return return_value

s14adc_ctypes = _library_cl.s14adc
s14adc_ctypes.restype = None
s14adc_ctypes.argtypes = [c_double, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def s14adc(x, n, m, ans, fail):
    """
    Scaled derivatives of psi(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14adc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ans
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ans
    """
    from .util import nag_double_type_check_and_cast
    _ans = nag_double_type_check_and_cast('s14adc', ans, 'ans')
    s14adc_ctypes(x, n, m, _ans, fail)

s14aec_ctypes = _library_cl.s14aec
s14aec_ctypes.restype = c_double
s14aec_ctypes.argtypes = [c_double, Integer, POINTER(NagError)]


def s14aec(x, k, fail):
    """
    Derivative of the psi function psi(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14aec.html
    """
    return_value = s14aec_ctypes(x, k, fail)
    return return_value

s14afc_ctypes = _library_cl.s14afc
s14afc_ctypes.restype = Complex
s14afc_ctypes.argtypes = [Nag_Complex_Type, Integer, POINTER(NagError)]


def s14afc(z, k, fail):
    """
    Derivative of the psi function psi(z).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14afc.html
    """
    return_value = s14afc_ctypes(z, k, fail)
    return return_value

s14agc_ctypes = _library_cl.s14agc
s14agc_ctypes.restype = Complex
s14agc_ctypes.argtypes = [Nag_Complex_Type, POINTER(NagError)]


def s14agc(z, fail):
    """
    Logarithm of the gamma function ln(Gamma)(z), complex argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14agc.html
    """
    return_value = s14agc_ctypes(z, fail)
    return return_value

s14ahc_ctypes = _library_cl.s14ahc
s14ahc_ctypes.restype = c_double
s14ahc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s14ahc(x, fail):
    """
    Scaled log gamma function ln(G(x)), where G(x) = Gamma(x+1)/(x/e)**x.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14ahc.html
    """
    return_value = s14ahc_ctypes(x, fail)
    return return_value

s14bac_ctypes = _library_cl.s14bac
s14bac_ctypes.restype = None
s14bac_ctypes.argtypes = [c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s14bac(a, x, tol, p, q, fail):
    """
    Incomplete gamma functions P(a,x) and Q(a,x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        p, q
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, q
    """
    from .util import nag_double_type_check_and_cast
    _p = nag_double_type_check_and_cast('s14bac', p, 'p')
    _q = nag_double_type_check_and_cast('s14bac', q, 'q')
    s14bac_ctypes(a, x, tol, _p, _q, fail)

s14cbc_ctypes = _library_cl.s14cbc
s14cbc_ctypes.restype = c_double
s14cbc_ctypes.argtypes = [c_double, c_double, POINTER(NagError)]


def s14cbc(a, b, fail):
    """
    Logarithm of the beta function ln(B)(a,b).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14cbc.html
    """
    return_value = s14cbc_ctypes(a, b, fail)
    return return_value

s14ccc_ctypes = _library_cl.s14ccc
s14ccc_ctypes.restype = None
s14ccc_ctypes.argtypes = [c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s14ccc(a, b, x, w, w1, fail):
    """
    Incomplete beta function I_x(a,b) and its complement 1-I_x.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s14ccc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        w, w1
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w, w1
    """
    from .util import nag_double_type_check_and_cast
    _w = nag_double_type_check_and_cast('s14ccc', w, 'w')
    _w1 = nag_double_type_check_and_cast('s14ccc', w1, 'w1')
    s14ccc_ctypes(a, b, x, _w, _w1, fail)

s15abc_ctypes = _library_cl.s15abc
s15abc_ctypes.restype = c_double
s15abc_ctypes.argtypes = [c_double]


def s15abc(x):
    """
    Cumulative Normal distribution function P(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s15abc.html
    """
    return_value = s15abc_ctypes(x)
    return return_value

s15acc_ctypes = _library_cl.s15acc
s15acc_ctypes.restype = c_double
s15acc_ctypes.argtypes = [c_double]


def s15acc(x):
    """
    Complement of cumulative Normal distribution function Q(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s15acc.html
    """
    return_value = s15acc_ctypes(x)
    return return_value

s15adc_ctypes = _library_cl.s15adc
s15adc_ctypes.restype = c_double
s15adc_ctypes.argtypes = [c_double]


def s15adc(x):
    """
    Complement of error function erfc(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s15adc.html
    """
    return_value = s15adc_ctypes(x)
    return return_value

s15aec_ctypes = _library_cl.s15aec
s15aec_ctypes.restype = c_double
s15aec_ctypes.argtypes = [c_double]


def s15aec(x):
    """
    Error function erf(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s15aec.html
    """
    return_value = s15aec_ctypes(x)
    return return_value

s15afc_ctypes = _library_cl.s15afc
s15afc_ctypes.restype = c_double
s15afc_ctypes.argtypes = [c_double]


def s15afc(x):
    """
    Dawson's integral.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s15afc.html
    """
    return_value = s15afc_ctypes(x)
    return return_value

s15agc_ctypes = _library_cl.s15agc
s15agc_ctypes.restype = c_double
s15agc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s15agc(x, fail):
    """
    Scaled complement of error function, erfcx(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s15agc.html
    """
    return_value = s15agc_ctypes(x, fail)
    return return_value

s15ddc_ctypes = _library_cl.s15ddc
s15ddc_ctypes.restype = Complex
s15ddc_ctypes.argtypes = [Nag_Complex_Type, POINTER(NagError)]


def s15ddc(z, fail):
    """
    Scaled complex complement of error function, exp(-z**2)erfc(-iz).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s15ddc.html
    """
    return_value = s15ddc_ctypes(z, fail)
    return return_value

s17acc_ctypes = _library_cl.s17acc
s17acc_ctypes.restype = c_double
s17acc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s17acc(x, fail):
    """
    Bessel function Y_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17acc.html
    """
    return_value = s17acc_ctypes(x, fail)
    return return_value

s17adc_ctypes = _library_cl.s17adc
s17adc_ctypes.restype = c_double
s17adc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s17adc(x, fail):
    """
    Bessel function Y_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17adc.html
    """
    return_value = s17adc_ctypes(x, fail)
    return return_value

s17aec_ctypes = _library_cl.s17aec
s17aec_ctypes.restype = c_double
s17aec_ctypes.argtypes = [c_double, POINTER(NagError)]


def s17aec(x, fail):
    """
    Bessel function J_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17aec.html
    """
    return_value = s17aec_ctypes(x, fail)
    return return_value

s17afc_ctypes = _library_cl.s17afc
s17afc_ctypes.restype = c_double
s17afc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s17afc(x, fail):
    """
    Bessel function J_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17afc.html
    """
    return_value = s17afc_ctypes(x, fail)
    return return_value

s17agc_ctypes = _library_cl.s17agc
s17agc_ctypes.restype = c_double
s17agc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s17agc(x, fail):
    """
    Airy function Ai(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17agc.html
    """
    return_value = s17agc_ctypes(x, fail)
    return return_value

s17ahc_ctypes = _library_cl.s17ahc
s17ahc_ctypes.restype = c_double
s17ahc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s17ahc(x, fail):
    """
    Airy function Bi(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17ahc.html
    """
    return_value = s17ahc_ctypes(x, fail)
    return return_value

s17ajc_ctypes = _library_cl.s17ajc
s17ajc_ctypes.restype = c_double
s17ajc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s17ajc(x, fail):
    """
    Airy function Ai'(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17ajc.html
    """
    return_value = s17ajc_ctypes(x, fail)
    return return_value

s17akc_ctypes = _library_cl.s17akc
s17akc_ctypes.restype = c_double
s17akc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s17akc(x, fail):
    """
    Airy function Bi'(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17akc.html
    """
    return_value = s17akc_ctypes(x, fail)
    return return_value

s17alc_ctypes = _library_cl.s17alc
s17alc_ctypes.restype = None
s17alc_ctypes.argtypes = [c_double, Integer, Integer, c_double, POINTER(c_double), POINTER(NagError)]


def s17alc(a, n, mode, rel, x, fail):
    """
    Zeros of Bessel functions J_alpha(x), J_alpha'(x), Y_alpha(x) or
    Y_alpha'(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17alc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17alc', x, 'x')
    s17alc_ctypes(a, n, mode, rel, _x, fail)

s17aqc_ctypes = _library_cl.s17aqc
s17aqc_ctypes.restype = None
s17aqc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s17aqc(n, x, f, ivalid, fail):
    """
    Bessel function vectorized Y_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17aqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17aqc', x, 'x')
    _f = nag_double_type_check_and_cast('s17aqc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s17aqc', ivalid, 'ivalid')
    s17aqc_ctypes(n, _x, _f, _ivalid, fail)

s17arc_ctypes = _library_cl.s17arc
s17arc_ctypes.restype = None
s17arc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s17arc(n, x, f, ivalid, fail):
    """
    Bessel function vectorized Y_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17arc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17arc', x, 'x')
    _f = nag_double_type_check_and_cast('s17arc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s17arc', ivalid, 'ivalid')
    s17arc_ctypes(n, _x, _f, _ivalid, fail)

s17asc_ctypes = _library_cl.s17asc
s17asc_ctypes.restype = None
s17asc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s17asc(n, x, f, ivalid, fail):
    """
    Bessel function vectorized J_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17asc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17asc', x, 'x')
    _f = nag_double_type_check_and_cast('s17asc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s17asc', ivalid, 'ivalid')
    s17asc_ctypes(n, _x, _f, _ivalid, fail)

s17atc_ctypes = _library_cl.s17atc
s17atc_ctypes.restype = None
s17atc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s17atc(n, x, f, ivalid, fail):
    """
    Bessel function vectorized J_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17atc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17atc', x, 'x')
    _f = nag_double_type_check_and_cast('s17atc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s17atc', ivalid, 'ivalid')
    s17atc_ctypes(n, _x, _f, _ivalid, fail)

s17auc_ctypes = _library_cl.s17auc
s17auc_ctypes.restype = None
s17auc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s17auc(n, x, f, ivalid, fail):
    """
    Airy function vectorized Ai(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17auc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17auc', x, 'x')
    _f = nag_double_type_check_and_cast('s17auc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s17auc', ivalid, 'ivalid')
    s17auc_ctypes(n, _x, _f, _ivalid, fail)

s17avc_ctypes = _library_cl.s17avc
s17avc_ctypes.restype = None
s17avc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s17avc(n, x, f, ivalid, fail):
    """
    Airy function vectorized Bi(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17avc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17avc', x, 'x')
    _f = nag_double_type_check_and_cast('s17avc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s17avc', ivalid, 'ivalid')
    s17avc_ctypes(n, _x, _f, _ivalid, fail)

s17awc_ctypes = _library_cl.s17awc
s17awc_ctypes.restype = None
s17awc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s17awc(n, x, f, ivalid, fail):
    """
    Derivatives of the Airy function, vectorized Ai'(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17awc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17awc', x, 'x')
    _f = nag_double_type_check_and_cast('s17awc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s17awc', ivalid, 'ivalid')
    s17awc_ctypes(n, _x, _f, _ivalid, fail)

s17axc_ctypes = _library_cl.s17axc
s17axc_ctypes.restype = None
s17axc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s17axc(n, x, f, ivalid, fail):
    """
    Derivatives of the Airy function, vectorized Bi'(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17axc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s17axc', x, 'x')
    _f = nag_double_type_check_and_cast('s17axc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s17axc', ivalid, 'ivalid')
    s17axc_ctypes(n, _x, _f, _ivalid, fail)

s17dcc_ctypes = _library_cl.s17dcc
s17dcc_ctypes.restype = None
s17dcc_ctypes.argtypes = [c_double, Nag_Complex_Type, Integer, Nag_ScaleResType, POINTER(Complex), POINTER(Integer), POINTER(NagError)]


def s17dcc(fnu, z, n, scal, cy, nz, fail):
    """
    Bessel functions Y_(nu+a)(z), real a >= 0, complex z, nu = 0,1,2,...
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cy, nz
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nz
    TypeError will be raised when the following arguments are not of data type
    Complex:
        cy
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _cy = nag_complex_type_check_and_cast('s17dcc', cy, 'cy')
    _nz = nag_integer_type_check_and_cast('s17dcc', nz, 'nz')
    s17dcc_ctypes(fnu, z, n, scal, _cy, _nz, fail)

s17dec_ctypes = _library_cl.s17dec
s17dec_ctypes.restype = None
s17dec_ctypes.argtypes = [c_double, Nag_Complex_Type, Integer, Nag_ScaleResType, POINTER(Complex), POINTER(Integer), POINTER(NagError)]


def s17dec(fnu, z, n, scal, cy, nz, fail):
    """
    Bessel functions J_(nu+a)(z), real a >= 0, complex z, nu = 0,1,2,...
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17dec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cy, nz
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nz
    TypeError will be raised when the following arguments are not of data type
    Complex:
        cy
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _cy = nag_complex_type_check_and_cast('s17dec', cy, 'cy')
    _nz = nag_integer_type_check_and_cast('s17dec', nz, 'nz')
    s17dec_ctypes(fnu, z, n, scal, _cy, _nz, fail)

s17dgc_ctypes = _library_cl.s17dgc
s17dgc_ctypes.restype = None
s17dgc_ctypes.argtypes = [Nag_FunType, Nag_Complex_Type, Nag_ScaleResType, POINTER(Complex), POINTER(Integer), POINTER(NagError)]


def s17dgc(deriv, z, scal, ai, nz, fail):
    """
    Airy functions Ai(z) and Ai'(z), complex z.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17dgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ai, nz
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nz
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ai
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ai = nag_complex_type_check_and_cast('s17dgc', ai, 'ai')
    _nz = nag_integer_type_check_and_cast('s17dgc', nz, 'nz')
    s17dgc_ctypes(deriv, z, scal, _ai, _nz, fail)

s17dhc_ctypes = _library_cl.s17dhc
s17dhc_ctypes.restype = None
s17dhc_ctypes.argtypes = [Nag_FunType, Nag_Complex_Type, Nag_ScaleResType, POINTER(Complex), POINTER(NagError)]


def s17dhc(deriv, z, scal, bi, fail):
    """
    Airy functions Bi(z) and Bi'(z), complex z.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17dhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        bi
    TypeError will be raised when the following arguments are not of data type
    Complex:
        bi
    """
    from .util import nag_complex_type_check_and_cast
    _bi = nag_complex_type_check_and_cast('s17dhc', bi, 'bi')
    s17dhc_ctypes(deriv, z, scal, _bi, fail)

s17dlc_ctypes = _library_cl.s17dlc
s17dlc_ctypes.restype = None
s17dlc_ctypes.argtypes = [Integer, c_double, Nag_Complex_Type, Integer, Nag_ScaleResType, POINTER(Complex), POINTER(Integer), POINTER(NagError)]


def s17dlc(m, fnu, z, n, scal, cy, nz, fail):
    """
    Hankel functions H_nu+a**(j)(z), j = 1,2, real a >= 0, complex z,
    nu=0,1,2,...
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s17dlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cy, nz
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nz
    TypeError will be raised when the following arguments are not of data type
    Complex:
        cy
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _cy = nag_complex_type_check_and_cast('s17dlc', cy, 'cy')
    _nz = nag_integer_type_check_and_cast('s17dlc', nz, 'nz')
    s17dlc_ctypes(m, fnu, z, n, scal, _cy, _nz, fail)

s18acc_ctypes = _library_cl.s18acc
s18acc_ctypes.restype = c_double
s18acc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s18acc(x, fail):
    """
    Modified Bessel function K_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18acc.html
    """
    return_value = s18acc_ctypes(x, fail)
    return return_value

s18adc_ctypes = _library_cl.s18adc
s18adc_ctypes.restype = c_double
s18adc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s18adc(x, fail):
    """
    Modified Bessel function K_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18adc.html
    """
    return_value = s18adc_ctypes(x, fail)
    return return_value

s18aec_ctypes = _library_cl.s18aec
s18aec_ctypes.restype = c_double
s18aec_ctypes.argtypes = [c_double, POINTER(NagError)]


def s18aec(x, fail):
    """
    Modified Bessel function I_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18aec.html
    """
    return_value = s18aec_ctypes(x, fail)
    return return_value

s18afc_ctypes = _library_cl.s18afc
s18afc_ctypes.restype = c_double
s18afc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s18afc(x, fail):
    """
    Modified Bessel function I_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18afc.html
    """
    return_value = s18afc_ctypes(x, fail)
    return return_value

s18aqc_ctypes = _library_cl.s18aqc
s18aqc_ctypes.restype = None
s18aqc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s18aqc(n, x, f, ivalid, fail):
    """
    Modified Bessel function vectorized K_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18aqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s18aqc', x, 'x')
    _f = nag_double_type_check_and_cast('s18aqc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s18aqc', ivalid, 'ivalid')
    s18aqc_ctypes(n, _x, _f, _ivalid, fail)

s18arc_ctypes = _library_cl.s18arc
s18arc_ctypes.restype = None
s18arc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s18arc(n, x, f, ivalid, fail):
    """
    Modified Bessel function vectorized K_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18arc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s18arc', x, 'x')
    _f = nag_double_type_check_and_cast('s18arc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s18arc', ivalid, 'ivalid')
    s18arc_ctypes(n, _x, _f, _ivalid, fail)

s18asc_ctypes = _library_cl.s18asc
s18asc_ctypes.restype = None
s18asc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s18asc(n, x, f, ivalid, fail):
    """
    Modified Bessel function vectorized I_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18asc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s18asc', x, 'x')
    _f = nag_double_type_check_and_cast('s18asc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s18asc', ivalid, 'ivalid')
    s18asc_ctypes(n, _x, _f, _ivalid, fail)

s18atc_ctypes = _library_cl.s18atc
s18atc_ctypes.restype = None
s18atc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s18atc(n, x, f, ivalid, fail):
    """
    Modified Bessel function vectorized I_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18atc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s18atc', x, 'x')
    _f = nag_double_type_check_and_cast('s18atc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s18atc', ivalid, 'ivalid')
    s18atc_ctypes(n, _x, _f, _ivalid, fail)

s18ccc_ctypes = _library_cl.s18ccc
s18ccc_ctypes.restype = c_double
s18ccc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s18ccc(x, fail):
    """
    Scaled modified Bessel function e**xK_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18ccc.html
    """
    return_value = s18ccc_ctypes(x, fail)
    return return_value

s18cdc_ctypes = _library_cl.s18cdc
s18cdc_ctypes.restype = c_double
s18cdc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s18cdc(x, fail):
    """
    Scaled modified Bessel function e**xK_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18cdc.html
    """
    return_value = s18cdc_ctypes(x, fail)
    return return_value

s18cec_ctypes = _library_cl.s18cec
s18cec_ctypes.restype = c_double
s18cec_ctypes.argtypes = [c_double]


def s18cec(x):
    """
    Scaled modified Bessel function e**(-abs(x))I_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18cec.html
    """
    return_value = s18cec_ctypes(x)
    return return_value

s18cfc_ctypes = _library_cl.s18cfc
s18cfc_ctypes.restype = c_double
s18cfc_ctypes.argtypes = [c_double]


def s18cfc(x):
    """
    Scaled modified Bessel function e**(-abs(x))I_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18cfc.html
    """
    return_value = s18cfc_ctypes(x)
    return return_value

s18cqc_ctypes = _library_cl.s18cqc
s18cqc_ctypes.restype = None
s18cqc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s18cqc(n, x, f, ivalid, fail):
    """
    Scaled modified Bessel function vectorized e**xK_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18cqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s18cqc', x, 'x')
    _f = nag_double_type_check_and_cast('s18cqc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s18cqc', ivalid, 'ivalid')
    s18cqc_ctypes(n, _x, _f, _ivalid, fail)

s18crc_ctypes = _library_cl.s18crc
s18crc_ctypes.restype = None
s18crc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s18crc(n, x, f, ivalid, fail):
    """
    Scaled modified Bessel function vectorized e**xK_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18crc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s18crc', x, 'x')
    _f = nag_double_type_check_and_cast('s18crc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s18crc', ivalid, 'ivalid')
    s18crc_ctypes(n, _x, _f, _ivalid, fail)

s18csc_ctypes = _library_cl.s18csc
s18csc_ctypes.restype = None
s18csc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s18csc(n, x, f, fail):
    """
    Scaled modified Bessel function vectorized e**(-abs(x))I_0(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18csc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s18csc', x, 'x')
    _f = nag_double_type_check_and_cast('s18csc', f, 'f')
    s18csc_ctypes(n, _x, _f, fail)

s18ctc_ctypes = _library_cl.s18ctc
s18ctc_ctypes.restype = None
s18ctc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s18ctc(n, x, f, fail):
    """
    Scaled modified Bessel function vectorized e**(-abs(x))I_1(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18ctc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s18ctc', x, 'x')
    _f = nag_double_type_check_and_cast('s18ctc', f, 'f')
    s18ctc_ctypes(n, _x, _f, fail)

s18dcc_ctypes = _library_cl.s18dcc
s18dcc_ctypes.restype = None
s18dcc_ctypes.argtypes = [c_double, Nag_Complex_Type, Integer, Nag_ScaleResType, POINTER(Complex), POINTER(Integer), POINTER(NagError)]


def s18dcc(fnu, z, n, scal, cy, nz, fail):
    """
    Modified Bessel functions K_(nu+a)(z), real a >= 0, complex z, nu =
    0,1,2,...
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cy, nz
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nz
    TypeError will be raised when the following arguments are not of data type
    Complex:
        cy
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _cy = nag_complex_type_check_and_cast('s18dcc', cy, 'cy')
    _nz = nag_integer_type_check_and_cast('s18dcc', nz, 'nz')
    s18dcc_ctypes(fnu, z, n, scal, _cy, _nz, fail)

s18dec_ctypes = _library_cl.s18dec
s18dec_ctypes.restype = None
s18dec_ctypes.argtypes = [c_double, Nag_Complex_Type, Integer, Nag_ScaleResType, POINTER(Complex), POINTER(Integer), POINTER(NagError)]


def s18dec(fnu, z, n, scal, cy, nz, fail):
    """
    Modified Bessel functions I_(nu+a)(z), real a >= 0, complex z, nu =
    0,1,2,...
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18dec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cy, nz
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nz
    TypeError will be raised when the following arguments are not of data type
    Complex:
        cy
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _cy = nag_complex_type_check_and_cast('s18dec', cy, 'cy')
    _nz = nag_integer_type_check_and_cast('s18dec', nz, 'nz')
    s18dec_ctypes(fnu, z, n, scal, _cy, _nz, fail)

s18ecc_ctypes = _library_cl.s18ecc
s18ecc_ctypes.restype = c_double
s18ecc_ctypes.argtypes = [c_double, Integer, POINTER(NagError)]


def s18ecc(x, nu, fail):
    """
    Scaled modified Bessel function e**(-x)I_(nu/4)(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18ecc.html
    """
    return_value = s18ecc_ctypes(x, nu, fail)
    return return_value

s18edc_ctypes = _library_cl.s18edc
s18edc_ctypes.restype = c_double
s18edc_ctypes.argtypes = [c_double, Integer, POINTER(NagError)]


def s18edc(x, nu, fail):
    """
    Scaled modified Bessel function e**xK_(nu/4)(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18edc.html
    """
    return_value = s18edc_ctypes(x, nu, fail)
    return return_value

s18eec_ctypes = _library_cl.s18eec
s18eec_ctypes.restype = c_double
s18eec_ctypes.argtypes = [c_double, Integer, POINTER(NagError)]


def s18eec(x, nu, fail):
    """
    Modified Bessel function I_(nu/4)(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18eec.html
    """
    return_value = s18eec_ctypes(x, nu, fail)
    return return_value

s18efc_ctypes = _library_cl.s18efc
s18efc_ctypes.restype = c_double
s18efc_ctypes.argtypes = [c_double, Integer, POINTER(NagError)]


def s18efc(x, nu, fail):
    """
    Modified Bessel function K_(nu/4)(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18efc.html
    """
    return_value = s18efc_ctypes(x, nu, fail)
    return return_value

s18egc_ctypes = _library_cl.s18egc
s18egc_ctypes.restype = None
s18egc_ctypes.argtypes = [c_double, Integer, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def s18egc(x, ia, ja, nl, b, fail):
    """
    Modified Bessel functions K_(alpha+n)(x) for real x > 0, selected values
    of alpha >= 0 and n = 0,1,...,N.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18egc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        b
    """
    from .util import nag_double_type_check_and_cast
    _b = nag_double_type_check_and_cast('s18egc', b, 'b')
    s18egc_ctypes(x, ia, ja, nl, _b, fail)

s18ehc_ctypes = _library_cl.s18ehc
s18ehc_ctypes.restype = None
s18ehc_ctypes.argtypes = [c_double, Integer, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def s18ehc(x, ia, ja, nl, b, fail):
    """
    Scaled modified Bessel functions e**xK_(alpha+n)(x) for real x > 0,
    selected values of alpha >= 0 and n = 0,1,...,N.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18ehc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        b
    """
    from .util import nag_double_type_check_and_cast
    _b = nag_double_type_check_and_cast('s18ehc', b, 'b')
    s18ehc_ctypes(x, ia, ja, nl, _b, fail)

s18ejc_ctypes = _library_cl.s18ejc
s18ejc_ctypes.restype = None
s18ejc_ctypes.argtypes = [c_double, c_double, Integer, POINTER(Complex), POINTER(NagError)]


def s18ejc(x, a, nl, b, fail):
    """
    Modified Bessel functions I_(alpha+n-1)(x) or I_(alpha-n+1)(x) for real
    x /= 0, non-negative alpha < 1 and n = 1,2,...,abs(N)+1.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18ejc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        b
    """
    from .util import nag_complex_type_check_and_cast
    _b = nag_complex_type_check_and_cast('s18ejc', b, 'b')
    s18ejc_ctypes(x, a, nl, _b, fail)

s18ekc_ctypes = _library_cl.s18ekc
s18ekc_ctypes.restype = None
s18ekc_ctypes.argtypes = [c_double, c_double, Integer, POINTER(Complex), POINTER(NagError)]


def s18ekc(x, a, nl, b, fail):
    """
    Bessel functions J_(alpha+n-1)(x) or J_(alpha-n+1)(x) for real x /= 0,
    non-negative alpha < 1 and n = 1,2,...,abs(N)+1.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18ekc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        b
    """
    from .util import nag_complex_type_check_and_cast
    _b = nag_complex_type_check_and_cast('s18ekc', b, 'b')
    s18ekc_ctypes(x, a, nl, _b, fail)

s18gkc_ctypes = _library_cl.s18gkc
s18gkc_ctypes.restype = None
s18gkc_ctypes.argtypes = [Nag_Complex_Type, c_double, Integer, POINTER(Complex), POINTER(NagError)]


def s18gkc(z, a, nl, b, fail):
    """
    Bessel function of the 1st kind J_(alpha (+/-) n)(z).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s18gkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        b
    """
    from .util import nag_complex_type_check_and_cast
    _b = nag_complex_type_check_and_cast('s18gkc', b, 'b')
    s18gkc_ctypes(z, a, nl, _b, fail)

s19aac_ctypes = _library_cl.s19aac
s19aac_ctypes.restype = c_double
s19aac_ctypes.argtypes = [c_double, POINTER(NagError)]


def s19aac(x, fail):
    """
    Kelvin function ber(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s19aac.html
    """
    return_value = s19aac_ctypes(x, fail)
    return return_value

s19abc_ctypes = _library_cl.s19abc
s19abc_ctypes.restype = c_double
s19abc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s19abc(x, fail):
    """
    Kelvin function bei(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s19abc.html
    """
    return_value = s19abc_ctypes(x, fail)
    return return_value

s19acc_ctypes = _library_cl.s19acc
s19acc_ctypes.restype = c_double
s19acc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s19acc(x, fail):
    """
    Kelvin function ker(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s19acc.html
    """
    return_value = s19acc_ctypes(x, fail)
    return return_value

s19adc_ctypes = _library_cl.s19adc
s19adc_ctypes.restype = c_double
s19adc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s19adc(x, fail):
    """
    Kelvin function kei(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s19adc.html
    """
    return_value = s19adc_ctypes(x, fail)
    return return_value

s19anc_ctypes = _library_cl.s19anc
s19anc_ctypes.restype = None
s19anc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s19anc(n, x, f, ivalid, fail):
    """
    Kelvin function vectorized ber(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s19anc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s19anc', x, 'x')
    _f = nag_double_type_check_and_cast('s19anc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s19anc', ivalid, 'ivalid')
    s19anc_ctypes(n, _x, _f, _ivalid, fail)

s19apc_ctypes = _library_cl.s19apc
s19apc_ctypes.restype = None
s19apc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s19apc(n, x, f, ivalid, fail):
    """
    Kelvin function vectorized bei(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s19apc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s19apc', x, 'x')
    _f = nag_double_type_check_and_cast('s19apc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s19apc', ivalid, 'ivalid')
    s19apc_ctypes(n, _x, _f, _ivalid, fail)

s19aqc_ctypes = _library_cl.s19aqc
s19aqc_ctypes.restype = None
s19aqc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s19aqc(n, x, f, ivalid, fail):
    """
    Kelvin function vectorized ker(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s19aqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s19aqc', x, 'x')
    _f = nag_double_type_check_and_cast('s19aqc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s19aqc', ivalid, 'ivalid')
    s19aqc_ctypes(n, _x, _f, _ivalid, fail)

s19arc_ctypes = _library_cl.s19arc
s19arc_ctypes.restype = None
s19arc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s19arc(n, x, f, ivalid, fail):
    """
    Kelvin function vectorized kei(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s19arc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, ivalid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalid
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('s19arc', x, 'x')
    _f = nag_double_type_check_and_cast('s19arc', f, 'f')
    _ivalid = nag_integer_type_check_and_cast('s19arc', ivalid, 'ivalid')
    s19arc_ctypes(n, _x, _f, _ivalid, fail)

s20acc_ctypes = _library_cl.s20acc
s20acc_ctypes.restype = c_double
s20acc_ctypes.argtypes = [c_double]


def s20acc(x):
    """
    Fresnel integral S(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s20acc.html
    """
    return_value = s20acc_ctypes(x)
    return return_value

s20adc_ctypes = _library_cl.s20adc
s20adc_ctypes.restype = c_double
s20adc_ctypes.argtypes = [c_double]


def s20adc(x):
    """
    Fresnel integral C(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s20adc.html
    """
    return_value = s20adc_ctypes(x)
    return return_value

s20aqc_ctypes = _library_cl.s20aqc
s20aqc_ctypes.restype = None
s20aqc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s20aqc(n, x, f, fail):
    """
    Fresnel integral vectorized S(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s20aqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s20aqc', x, 'x')
    _f = nag_double_type_check_and_cast('s20aqc', f, 'f')
    s20aqc_ctypes(n, _x, _f, fail)

s20arc_ctypes = _library_cl.s20arc
s20arc_ctypes.restype = None
s20arc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s20arc(n, x, f, fail):
    """
    Fresnel integral vectorized C(x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s20arc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s20arc', x, 'x')
    _f = nag_double_type_check_and_cast('s20arc', f, 'f')
    s20arc_ctypes(n, _x, _f, fail)

s21bac_ctypes = _library_cl.s21bac
s21bac_ctypes.restype = c_double
s21bac_ctypes.argtypes = [c_double, c_double, POINTER(NagError)]


def s21bac(x, y, fail):
    """
    Degenerate symmetrised elliptic integral of 1st kind R_C(x,y).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bac.html
    """
    return_value = s21bac_ctypes(x, y, fail)
    return return_value

s21bbc_ctypes = _library_cl.s21bbc
s21bbc_ctypes.restype = c_double
s21bbc_ctypes.argtypes = [c_double, c_double, c_double, POINTER(NagError)]


def s21bbc(x, y, z, fail):
    """
    Symmetrised elliptic integral of 1st kind R_F(x,y,z).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bbc.html
    """
    return_value = s21bbc_ctypes(x, y, z, fail)
    return return_value

s21bcc_ctypes = _library_cl.s21bcc
s21bcc_ctypes.restype = c_double
s21bcc_ctypes.argtypes = [c_double, c_double, c_double, POINTER(NagError)]


def s21bcc(x, y, z, fail):
    """
    Symmetrised elliptic integral of 2nd kind R_D(x,y,z).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bcc.html
    """
    return_value = s21bcc_ctypes(x, y, z, fail)
    return return_value

s21bdc_ctypes = _library_cl.s21bdc
s21bdc_ctypes.restype = c_double
s21bdc_ctypes.argtypes = [c_double, c_double, c_double, c_double, POINTER(NagError)]


def s21bdc(x, y, z, r, fail):
    """
    Symmetrised elliptic integral of 3rd kind R_J(x,y,z,r).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bdc.html
    """
    return_value = s21bdc_ctypes(x, y, z, r, fail)
    return return_value

s21bec_ctypes = _library_cl.s21bec
s21bec_ctypes.restype = c_double
s21bec_ctypes.argtypes = [c_double, c_double, POINTER(NagError)]


def s21bec(phi, dm, fail):
    """
    Elliptic integral of 1st kind, Legendre form, F(phi | m).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bec.html
    """
    return_value = s21bec_ctypes(phi, dm, fail)
    return return_value

s21bfc_ctypes = _library_cl.s21bfc
s21bfc_ctypes.restype = c_double
s21bfc_ctypes.argtypes = [c_double, c_double, POINTER(NagError)]


def s21bfc(phi, dm, fail):
    """
    Elliptic integral of 2nd kind, Legendre form, E(phi | m).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bfc.html
    """
    return_value = s21bfc_ctypes(phi, dm, fail)
    return return_value

s21bgc_ctypes = _library_cl.s21bgc
s21bgc_ctypes.restype = c_double
s21bgc_ctypes.argtypes = [c_double, c_double, c_double, POINTER(NagError)]


def s21bgc(dn, phi, dm, fail):
    """
    Elliptic integral of 3rd kind, Legendre form, Pi(n;phi | m).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bgc.html
    """
    return_value = s21bgc_ctypes(dn, phi, dm, fail)
    return return_value

s21bhc_ctypes = _library_cl.s21bhc
s21bhc_ctypes.restype = c_double
s21bhc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s21bhc(dm, fail):
    """
    Complete elliptic integral of 1st kind, Legendre form, K(m).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bhc.html
    """
    return_value = s21bhc_ctypes(dm, fail)
    return return_value

s21bjc_ctypes = _library_cl.s21bjc
s21bjc_ctypes.restype = c_double
s21bjc_ctypes.argtypes = [c_double, POINTER(NagError)]


def s21bjc(dm, fail):
    """
    Complete elliptic integral of 2nd kind, Legendre form, E(m).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21bjc.html
    """
    return_value = s21bjc_ctypes(dm, fail)
    return return_value

s21cac_ctypes = _library_cl.s21cac
s21cac_ctypes.restype = None
s21cac_ctypes.argtypes = [c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s21cac(u, m, sn, cn, dn, fail):
    """
    Jacobian elliptic functions sn, cn and dn of real argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        sn, cn, dn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        sn, cn, dn
    """
    from .util import nag_double_type_check_and_cast
    _sn = nag_double_type_check_and_cast('s21cac', sn, 'sn')
    _cn = nag_double_type_check_and_cast('s21cac', cn, 'cn')
    _dn = nag_double_type_check_and_cast('s21cac', dn, 'dn')
    s21cac_ctypes(u, m, _sn, _cn, _dn, fail)

s21cbc_ctypes = _library_cl.s21cbc
s21cbc_ctypes.restype = None
s21cbc_ctypes.argtypes = [Nag_Complex_Type, c_double, POINTER(Complex), POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def s21cbc(z, ak2, sn, cn, dn, fail):
    """
    Jacobian elliptic functions sn, cn and dn of complex argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        sn, cn, dn
    TypeError will be raised when the following arguments are not of data type
    Complex:
        sn, cn, dn
    """
    from .util import nag_complex_type_check_and_cast
    _sn = nag_complex_type_check_and_cast('s21cbc', sn, 'sn')
    _cn = nag_complex_type_check_and_cast('s21cbc', cn, 'cn')
    _dn = nag_complex_type_check_and_cast('s21cbc', dn, 'dn')
    s21cbc_ctypes(z, ak2, _sn, _cn, _dn, fail)

s21ccc_ctypes = _library_cl.s21ccc
s21ccc_ctypes.restype = c_double
s21ccc_ctypes.argtypes = [Integer, c_double, c_double, POINTER(NagError)]


def s21ccc(k, x, q, fail):
    """
    Jacobian theta functions with real arguments.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21ccc.html
    """
    return_value = s21ccc_ctypes(k, x, q, fail)
    return return_value

s21dac_ctypes = _library_cl.s21dac
s21dac_ctypes.restype = Complex
s21dac_ctypes.argtypes = [Nag_Complex_Type, c_double, c_double, c_double, POINTER(NagError)]


def s21dac(z, akp, a, b, fail):
    """
    Elliptic integrals of the second kind with complex arguments.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s21dac.html
    """
    return_value = s21dac_ctypes(z, akp, a, b, fail)
    return return_value

s22aac_ctypes = _library_cl.s22aac
s22aac_ctypes.restype = None
s22aac_ctypes.argtypes = [Integer, c_double, Integer, Integer, POINTER(c_double), POINTER(NagError)]


def s22aac(mode, x, m, nl, p, fail):
    """
    Legendre and associated Legendre functions of the first kind with real
    arguments.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s22aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p
    """
    from .util import nag_double_type_check_and_cast
    _p = nag_double_type_check_and_cast('s22aac', p, 'p')
    s22aac_ctypes(mode, x, m, nl, _p, fail)

s22bac_ctypes = _library_cl.s22bac
s22bac_ctypes.restype = None
s22bac_ctypes.argtypes = [c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s22bac(a, b, x, m, fail):
    """
    Real confluent hypergeometric function 1F1(a;b;x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s22bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        m
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        m
    """
    from .util import nag_double_type_check_and_cast
    _m = nag_double_type_check_and_cast('s22bac', m, 'm')
    s22bac_ctypes(a, b, x, _m, fail)

s22bbc_ctypes = _library_cl.s22bbc
s22bbc_ctypes.restype = None
s22bbc_ctypes.argtypes = [c_double, c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s22bbc(ani, adr, bni, bdr, x, frm, scm, fail):
    """
    Real confluent hypergeometric function 1F1(a;b;x) in scaled form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s22bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        frm, scm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        frm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        scm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _frm = nag_double_type_check_and_cast('s22bbc', frm, 'frm')
    _scm = nag_integer_type_check_and_cast('s22bbc', scm, 'scm')
    s22bbc_ctypes(ani, adr, bni, bdr, x, _frm, _scm, fail)

s22bec_ctypes = _library_cl.s22bec
s22bec_ctypes.restype = c_double
s22bec_ctypes.argtypes = [c_double, c_double, c_double, c_double, POINTER(NagError)]


def s22bec(a, b, c, x, fail):
    """
    Real Gauss hypergeometric function 2F1(a,b;c;x).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s22bec.html
    """
    return_value = s22bec_ctypes(a, b, c, x, fail)
    return return_value

s22bfc_ctypes = _library_cl.s22bfc
s22bfc_ctypes.restype = None
s22bfc_ctypes.argtypes = [c_double, c_double, c_double, c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def s22bfc(ani, adr, bni, bdr, cni, cdr, x, frf, scf, fail):
    """
    Real Gauss hypergeometric function 2F1(a,b;c;x) in scaled form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s22bfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        frf, scf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        frf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        scf
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _frf = nag_double_type_check_and_cast('s22bfc', frf, 'frf')
    _scf = nag_integer_type_check_and_cast('s22bfc', scf, 'scf')
    s22bfc_ctypes(ani, adr, bni, bdr, cni, cdr, x, _frf, _scf, fail)

s30aac_ctypes = _library_cl.s30aac
s30aac_ctypes.restype = None
s30aac_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30aac(order, option, m, n, x, s, t, sigma, r, q, p, fail):
    """
    Black--Scholes--Merton option pricing formula.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30aac', x, 'x')
    _t = nag_double_type_check_and_cast('s30aac', t, 't')
    _p = nag_double_type_check_and_cast('s30aac', p, 'p')
    s30aac_ctypes(order, option, m, n, _x, s, _t, sigma, r, q, _p, fail)

s30abc_ctypes = _library_cl.s30abc
s30abc_ctypes.restype = None
s30abc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s30abc(order, option, m, n, x, s, t, sigma, r, q, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma, fail):
    """
    Black--Scholes--Merton option pricing formula with Greeks.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30abc', x, 'x')
    _t = nag_double_type_check_and_cast('s30abc', t, 't')
    _p = nag_double_type_check_and_cast('s30abc', p, 'p')
    _delta = nag_double_type_check_and_cast('s30abc', delta, 'delta')
    _gamma = nag_double_type_check_and_cast('s30abc', gamma, 'gamma')
    _vega = nag_double_type_check_and_cast('s30abc', vega, 'vega')
    _theta = nag_double_type_check_and_cast('s30abc', theta, 'theta')
    _rho = nag_double_type_check_and_cast('s30abc', rho, 'rho')
    _crho = nag_double_type_check_and_cast('s30abc', crho, 'crho')
    _vanna = nag_double_type_check_and_cast('s30abc', vanna, 'vanna')
    _charm = nag_double_type_check_and_cast('s30abc', charm, 'charm')
    _speed = nag_double_type_check_and_cast('s30abc', speed, 'speed')
    _colour = nag_double_type_check_and_cast('s30abc', colour, 'colour')
    _zomma = nag_double_type_check_and_cast('s30abc', zomma, 'zomma')
    _vomma = nag_double_type_check_and_cast('s30abc', vomma, 'vomma')
    s30abc_ctypes(order, option, m, n, _x, s, _t, sigma, r, q, _p, _delta, _gamma, _vega, _theta, _rho, _crho, _vanna, _charm, _speed, _colour, _zomma, _vomma, fail)

s30bac_ctypes = _library_cl.s30bac
s30bac_ctypes.restype = None
s30bac_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30bac(order, option, m, n, sm, s, t, sigma, r, q, p, fail):
    """
    Floating-strike lookback option pricing formula in the Black-Scholes-
    Merton model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        sm, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        sm, t, p
    """
    from .util import nag_double_type_check_and_cast
    _sm = nag_double_type_check_and_cast('s30bac', sm, 'sm')
    _t = nag_double_type_check_and_cast('s30bac', t, 't')
    _p = nag_double_type_check_and_cast('s30bac', p, 'p')
    s30bac_ctypes(order, option, m, n, _sm, s, _t, sigma, r, q, _p, fail)

s30bbc_ctypes = _library_cl.s30bbc
s30bbc_ctypes.restype = None
s30bbc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s30bbc(order, option, m, n, sm, s, t, sigma, r, q, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma, fail):
    """
    Floating-strike lookback option pricing formula with Greeks in the
    Black-Scholes-Merton model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        sm, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        sm, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    """
    from .util import nag_double_type_check_and_cast
    _sm = nag_double_type_check_and_cast('s30bbc', sm, 'sm')
    _t = nag_double_type_check_and_cast('s30bbc', t, 't')
    _p = nag_double_type_check_and_cast('s30bbc', p, 'p')
    _delta = nag_double_type_check_and_cast('s30bbc', delta, 'delta')
    _gamma = nag_double_type_check_and_cast('s30bbc', gamma, 'gamma')
    _vega = nag_double_type_check_and_cast('s30bbc', vega, 'vega')
    _theta = nag_double_type_check_and_cast('s30bbc', theta, 'theta')
    _rho = nag_double_type_check_and_cast('s30bbc', rho, 'rho')
    _crho = nag_double_type_check_and_cast('s30bbc', crho, 'crho')
    _vanna = nag_double_type_check_and_cast('s30bbc', vanna, 'vanna')
    _charm = nag_double_type_check_and_cast('s30bbc', charm, 'charm')
    _speed = nag_double_type_check_and_cast('s30bbc', speed, 'speed')
    _colour = nag_double_type_check_and_cast('s30bbc', colour, 'colour')
    _zomma = nag_double_type_check_and_cast('s30bbc', zomma, 'zomma')
    _vomma = nag_double_type_check_and_cast('s30bbc', vomma, 'vomma')
    s30bbc_ctypes(order, option, m, n, _sm, s, _t, sigma, r, q, _p, _delta, _gamma, _vega, _theta, _rho, _crho, _vanna, _charm, _speed, _colour, _zomma, _vomma, fail)

s30cac_ctypes = _library_cl.s30cac
s30cac_ctypes.restype = None
s30cac_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30cac(order, option, m, n, x, s, k, t, sigma, r, q, p, fail):
    """
    Binary option, cash-or-nothing pricing formula.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30cac', x, 'x')
    _t = nag_double_type_check_and_cast('s30cac', t, 't')
    _p = nag_double_type_check_and_cast('s30cac', p, 'p')
    s30cac_ctypes(order, option, m, n, _x, s, k, _t, sigma, r, q, _p, fail)

s30cbc_ctypes = _library_cl.s30cbc
s30cbc_ctypes.restype = None
s30cbc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s30cbc(order, option, m, n, x, s, k, t, sigma, r, q, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma, fail):
    """
    Binary option, cash-or-nothing pricing formula with Greeks.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30cbc', x, 'x')
    _t = nag_double_type_check_and_cast('s30cbc', t, 't')
    _p = nag_double_type_check_and_cast('s30cbc', p, 'p')
    _delta = nag_double_type_check_and_cast('s30cbc', delta, 'delta')
    _gamma = nag_double_type_check_and_cast('s30cbc', gamma, 'gamma')
    _vega = nag_double_type_check_and_cast('s30cbc', vega, 'vega')
    _theta = nag_double_type_check_and_cast('s30cbc', theta, 'theta')
    _rho = nag_double_type_check_and_cast('s30cbc', rho, 'rho')
    _crho = nag_double_type_check_and_cast('s30cbc', crho, 'crho')
    _vanna = nag_double_type_check_and_cast('s30cbc', vanna, 'vanna')
    _charm = nag_double_type_check_and_cast('s30cbc', charm, 'charm')
    _speed = nag_double_type_check_and_cast('s30cbc', speed, 'speed')
    _colour = nag_double_type_check_and_cast('s30cbc', colour, 'colour')
    _zomma = nag_double_type_check_and_cast('s30cbc', zomma, 'zomma')
    _vomma = nag_double_type_check_and_cast('s30cbc', vomma, 'vomma')
    s30cbc_ctypes(order, option, m, n, _x, s, k, _t, sigma, r, q, _p, _delta, _gamma, _vega, _theta, _rho, _crho, _vanna, _charm, _speed, _colour, _zomma, _vomma, fail)

s30ccc_ctypes = _library_cl.s30ccc
s30ccc_ctypes.restype = None
s30ccc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30ccc(order, option, m, n, x, s, t, sigma, r, q, p, fail):
    """
    Binary option, asset-or-nothing pricing formula.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30ccc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30ccc', x, 'x')
    _t = nag_double_type_check_and_cast('s30ccc', t, 't')
    _p = nag_double_type_check_and_cast('s30ccc', p, 'p')
    s30ccc_ctypes(order, option, m, n, _x, s, _t, sigma, r, q, _p, fail)

s30cdc_ctypes = _library_cl.s30cdc
s30cdc_ctypes.restype = None
s30cdc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s30cdc(order, option, m, n, x, s, t, sigma, r, q, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma, fail):
    """
    Binary option, asset-or-nothing pricing formula with Greeks.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30cdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30cdc', x, 'x')
    _t = nag_double_type_check_and_cast('s30cdc', t, 't')
    _p = nag_double_type_check_and_cast('s30cdc', p, 'p')
    _delta = nag_double_type_check_and_cast('s30cdc', delta, 'delta')
    _gamma = nag_double_type_check_and_cast('s30cdc', gamma, 'gamma')
    _vega = nag_double_type_check_and_cast('s30cdc', vega, 'vega')
    _theta = nag_double_type_check_and_cast('s30cdc', theta, 'theta')
    _rho = nag_double_type_check_and_cast('s30cdc', rho, 'rho')
    _crho = nag_double_type_check_and_cast('s30cdc', crho, 'crho')
    _vanna = nag_double_type_check_and_cast('s30cdc', vanna, 'vanna')
    _charm = nag_double_type_check_and_cast('s30cdc', charm, 'charm')
    _speed = nag_double_type_check_and_cast('s30cdc', speed, 'speed')
    _colour = nag_double_type_check_and_cast('s30cdc', colour, 'colour')
    _zomma = nag_double_type_check_and_cast('s30cdc', zomma, 'zomma')
    _vomma = nag_double_type_check_and_cast('s30cdc', vomma, 'vomma')
    s30cdc_ctypes(order, option, m, n, _x, s, _t, sigma, r, q, _p, _delta, _gamma, _vega, _theta, _rho, _crho, _vanna, _charm, _speed, _colour, _zomma, _vomma, fail)

s30fac_ctypes = _library_cl.s30fac
s30fac_ctypes.restype = None
s30fac_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Nag_Barrier, Integer, Integer, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30fac(order, option, itype, m, n, x, s, h, k, t, sigma, r, q, p, fail):
    """
    Standard barrier option pricing formula.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30fac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30fac', x, 'x')
    _t = nag_double_type_check_and_cast('s30fac', t, 't')
    _p = nag_double_type_check_and_cast('s30fac', p, 'p')
    s30fac_ctypes(order, option, itype, m, n, _x, s, h, k, _t, sigma, r, q, _p, fail)

s30jac_ctypes = _library_cl.s30jac
s30jac_ctypes.restype = None
s30jac_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30jac(order, option, m, n, x, s, t, sigma, r, _lambda, jvol, p, fail):
    """
    Jump-diffusion, Merton's model, option pricing formula.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30jac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30jac', x, 'x')
    _t = nag_double_type_check_and_cast('s30jac', t, 't')
    _p = nag_double_type_check_and_cast('s30jac', p, 'p')
    s30jac_ctypes(order, option, m, n, _x, s, _t, sigma, r, _lambda, jvol, _p, fail)

s30jbc_ctypes = _library_cl.s30jbc
s30jbc_ctypes.restype = None
s30jbc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s30jbc(order, option, m, n, x, s, t, sigma, r, _lambda, jvol, p, delta, gamma, vega, theta, rho, vanna, charm, speed, colour, zomma, vomma, fail):
    """
    Jump-diffusion, Merton's model, option pricing formula with Greeks.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30jbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p, delta, gamma, vega, theta, rho, vanna, charm, speed, colour, zomma, vomma
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p, delta, gamma, vega, theta, rho, vanna, charm, speed, colour, zomma, vomma
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30jbc', x, 'x')
    _t = nag_double_type_check_and_cast('s30jbc', t, 't')
    _p = nag_double_type_check_and_cast('s30jbc', p, 'p')
    _delta = nag_double_type_check_and_cast('s30jbc', delta, 'delta')
    _gamma = nag_double_type_check_and_cast('s30jbc', gamma, 'gamma')
    _vega = nag_double_type_check_and_cast('s30jbc', vega, 'vega')
    _theta = nag_double_type_check_and_cast('s30jbc', theta, 'theta')
    _rho = nag_double_type_check_and_cast('s30jbc', rho, 'rho')
    _vanna = nag_double_type_check_and_cast('s30jbc', vanna, 'vanna')
    _charm = nag_double_type_check_and_cast('s30jbc', charm, 'charm')
    _speed = nag_double_type_check_and_cast('s30jbc', speed, 'speed')
    _colour = nag_double_type_check_and_cast('s30jbc', colour, 'colour')
    _zomma = nag_double_type_check_and_cast('s30jbc', zomma, 'zomma')
    _vomma = nag_double_type_check_and_cast('s30jbc', vomma, 'vomma')
    s30jbc_ctypes(order, option, m, n, _x, s, _t, sigma, r, _lambda, jvol, _p, _delta, _gamma, _vega, _theta, _rho, _vanna, _charm, _speed, _colour, _zomma, _vomma, fail)

s30nac_ctypes = _library_cl.s30nac
s30nac_ctypes.restype = None
s30nac_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, c_double, c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30nac(order, option, m, n, x, s, t, sigmav, kappa, corr, var0, eta, grisk, r, q, p, fail):
    """
    Heston's model option pricing formula.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30nac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30nac', x, 'x')
    _t = nag_double_type_check_and_cast('s30nac', t, 't')
    _p = nag_double_type_check_and_cast('s30nac', p, 'p')
    s30nac_ctypes(order, option, m, n, _x, s, _t, sigmav, kappa, corr, var0, eta, grisk, r, q, _p, fail)

s30nbc_ctypes = _library_cl.s30nbc
s30nbc_ctypes.restype = None
s30nbc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, c_double, c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s30nbc(order, option, m, n, x, s, t, sigmav, kappa, corr, var0, eta, grisk, r, q, p, delta, gamma, vega, theta, rho, vanna, charm, speed, zomma, vomma, fail):
    """
    Heston's model option pricing formula with Greeks.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30nbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p, delta, gamma, vega, theta, rho, vanna, charm, speed, zomma, vomma
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p, delta, gamma, vega, theta, rho, vanna, charm, speed, zomma, vomma
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30nbc', x, 'x')
    _t = nag_double_type_check_and_cast('s30nbc', t, 't')
    _p = nag_double_type_check_and_cast('s30nbc', p, 'p')
    _delta = nag_double_type_check_and_cast('s30nbc', delta, 'delta')
    _gamma = nag_double_type_check_and_cast('s30nbc', gamma, 'gamma')
    _vega = nag_double_type_check_and_cast('s30nbc', vega, 'vega')
    _theta = nag_double_type_check_and_cast('s30nbc', theta, 'theta')
    _rho = nag_double_type_check_and_cast('s30nbc', rho, 'rho')
    _vanna = nag_double_type_check_and_cast('s30nbc', vanna, 'vanna')
    _charm = nag_double_type_check_and_cast('s30nbc', charm, 'charm')
    _speed = nag_double_type_check_and_cast('s30nbc', speed, 'speed')
    _zomma = nag_double_type_check_and_cast('s30nbc', zomma, 'zomma')
    _vomma = nag_double_type_check_and_cast('s30nbc', vomma, 'vomma')
    s30nbc_ctypes(order, option, m, n, _x, s, _t, sigmav, kappa, corr, var0, eta, grisk, r, q, _p, _delta, _gamma, _vega, _theta, _rho, _vanna, _charm, _speed, _zomma, _vomma, fail)

s30ncc_ctypes = _library_cl.s30ncc
s30ncc_ctypes.restype = None
s30ncc_ctypes.argtypes = [Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, c_double, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(NagError)]


def s30ncc(option, m, numts, x, fwd, disc, ts, t, alpha, _lambda, corr, sigmat, var0, p, fail):
    """
    Heston's model option pricing with term structure.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30ncc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, ts, alpha, _lambda, corr, sigmat, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, ts, alpha, _lambda, corr, sigmat, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30ncc', x, 'x')
    _ts = nag_double_type_check_and_cast('s30ncc', ts, 'ts')
    _alpha = nag_double_type_check_and_cast('s30ncc', alpha, 'alpha')
    __lambda = nag_double_type_check_and_cast('s30ncc', _lambda, '_lambda')
    _corr = nag_double_type_check_and_cast('s30ncc', corr, 'corr')
    _sigmat = nag_double_type_check_and_cast('s30ncc', sigmat, 'sigmat')
    _p = nag_double_type_check_and_cast('s30ncc', p, 'p')
    s30ncc_ctypes(option, m, numts, _x, fwd, disc, _ts, t, _alpha, __lambda, _corr, _sigmat, var0, _p, fail)

s30qcc_ctypes = _library_cl.s30qcc
s30qcc_ctypes.restype = None
s30qcc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30qcc(order, option, m, n, x, s, t, sigma, r, q, p, fail):
    """
    American option, Bjerksund and Stensland pricing formula.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30qcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30qcc', x, 'x')
    _t = nag_double_type_check_and_cast('s30qcc', t, 't')
    _p = nag_double_type_check_and_cast('s30qcc', p, 'p')
    s30qcc_ctypes(order, option, m, n, _x, s, _t, sigma, r, q, _p, fail)

s30sac_ctypes = _library_cl.s30sac
s30sac_ctypes.restype = None
s30sac_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(NagError)]


def s30sac(order, option, m, n, x, s, t, sigma, r, b, p, fail):
    """
    Asian option, geometric continuous average rate pricing formula.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30sac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30sac', x, 'x')
    _t = nag_double_type_check_and_cast('s30sac', t, 't')
    _p = nag_double_type_check_and_cast('s30sac', p, 'p')
    s30sac_ctypes(order, option, m, n, _x, s, _t, sigma, r, b, _p, fail)

s30sbc_ctypes = _library_cl.s30sbc
s30sbc_ctypes.restype = None
s30sbc_ctypes.argtypes = [Nag_OrderType, Nag_CallPut, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def s30sbc(order, option, m, n, x, s, t, sigma, r, b, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma, fail):
    """
    Asian option, geometric continuous average rate pricing formula with
    Greeks.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/s/s30sbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, t, p, delta, gamma, vega, theta, rho, crho, vanna, charm, speed, colour, zomma, vomma
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('s30sbc', x, 'x')
    _t = nag_double_type_check_and_cast('s30sbc', t, 't')
    _p = nag_double_type_check_and_cast('s30sbc', p, 'p')
    _delta = nag_double_type_check_and_cast('s30sbc', delta, 'delta')
    _gamma = nag_double_type_check_and_cast('s30sbc', gamma, 'gamma')
    _vega = nag_double_type_check_and_cast('s30sbc', vega, 'vega')
    _theta = nag_double_type_check_and_cast('s30sbc', theta, 'theta')
    _rho = nag_double_type_check_and_cast('s30sbc', rho, 'rho')
    _crho = nag_double_type_check_and_cast('s30sbc', crho, 'crho')
    _vanna = nag_double_type_check_and_cast('s30sbc', vanna, 'vanna')
    _charm = nag_double_type_check_and_cast('s30sbc', charm, 'charm')
    _speed = nag_double_type_check_and_cast('s30sbc', speed, 'speed')
    _colour = nag_double_type_check_and_cast('s30sbc', colour, 'colour')
    _zomma = nag_double_type_check_and_cast('s30sbc', zomma, 'zomma')
    _vomma = nag_double_type_check_and_cast('s30sbc', vomma, 'vomma')
    s30sbc_ctypes(order, option, m, n, _x, s, _t, sigma, r, b, _p, _delta, _gamma, _vega, _theta, _rho, _crho, _vanna, _charm, _speed, _colour, _zomma, _vomma, fail)

nag_shifted_log = s01bac
nag_tanh = s10aac
nag_sinh = s10abc
nag_cosh = s10acc
nag_arctanh = s11aac
nag_arcsinh = s11abc
nag_arccosh = s11acc
nag_exp_integral = s13aac
nag_cos_integral = s13acc
nag_sin_integral = s13adc
nag_gamma = s14aac
nag_log_gamma = s14abc
nag_polygamma_fun = s14acc
nag_polygamma_deriv = s14adc
nag_real_polygamma = s14aec
nag_complex_polygamma = s14afc
nag_complex_log_gamma = s14agc
nag_scaled_log_gamma = s14ahc
nag_incomplete_gamma = s14bac
nag_log_beta = s14cbc
nag_incomplete_beta = s14ccc
nag_cumul_normal = s15abc
nag_cumul_normal_complem = s15acc
nag_erfc = s15adc
nag_erf = s15aec
nag_dawson = s15afc
nag_erfcx = s15agc
nag_complex_erfc = s15ddc
nag_bessel_y0 = s17acc
nag_bessel_y1 = s17adc
nag_bessel_j0 = s17aec
nag_bessel_j1 = s17afc
nag_airy_ai = s17agc
nag_airy_bi = s17ahc
nag_airy_ai_deriv = s17ajc
nag_airy_bi_deriv = s17akc
nag_bessel_zeros = s17alc
nag_bessel_y0_vector = s17aqc
nag_bessel_y1_vector = s17arc
nag_bessel_j0_vector = s17asc
nag_bessel_j1_vector = s17atc
nag_airy_ai_vector = s17auc
nag_airy_bi_vector = s17avc
nag_airy_ai_deriv_vector = s17awc
nag_airy_bi_deriv_vector = s17axc
nag_complex_bessel_y = s17dcc
nag_complex_bessel_j = s17dec
nag_complex_airy_ai = s17dgc
nag_complex_airy_bi = s17dhc
nag_complex_hankel = s17dlc
nag_bessel_k0 = s18acc
nag_bessel_k1 = s18adc
nag_bessel_i0 = s18aec
nag_bessel_i1 = s18afc
nag_bessel_k0_vector = s18aqc
nag_bessel_k1_vector = s18arc
nag_bessel_i0_vector = s18asc
nag_bessel_i1_vector = s18atc
nag_bessel_k0_scaled = s18ccc
nag_bessel_k1_scaled = s18cdc
nag_bessel_i0_scaled = s18cec
nag_bessel_i1_scaled = s18cfc
nag_bessel_k0_scaled_vector = s18cqc
nag_bessel_k1_scaled_vector = s18crc
nag_bessel_i0_scaled_vector = s18csc
nag_bessel_i1_scaled_vector = s18ctc
nag_complex_bessel_k = s18dcc
nag_complex_bessel_i = s18dec
nag_bessel_i_nu_scaled = s18ecc
nag_bessel_k_nu_scaled = s18edc
nag_bessel_i_nu = s18eec
nag_bessel_k_nu = s18efc
nag_bessel_k_alpha = s18egc
nag_bessel_k_alpha_scaled = s18ehc
nag_bessel_i_alpha = s18ejc
nag_bessel_j_alpha = s18ekc
nag_complex_bessel_j_seq = s18gkc
nag_kelvin_ber = s19aac
nag_kelvin_bei = s19abc
nag_kelvin_ker = s19acc
nag_kelvin_kei = s19adc
nag_kelvin_ber_vector = s19anc
nag_kelvin_bei_vector = s19apc
nag_kelvin_ker_vector = s19aqc
nag_kelvin_kei_vector = s19arc
nag_fresnel_s = s20acc
nag_fresnel_c = s20adc
nag_fresnel_s_vector = s20aqc
nag_fresnel_c_vector = s20arc
nag_elliptic_integral_rc = s21bac
nag_elliptic_integral_rf = s21bbc
nag_elliptic_integral_rd = s21bcc
nag_elliptic_integral_rj = s21bdc
nag_elliptic_integral_F = s21bec
nag_elliptic_integral_E = s21bfc
nag_elliptic_integral_pi = s21bgc
nag_elliptic_integral_complete_K = s21bhc
nag_elliptic_integral_complete_E = s21bjc
nag_real_jacobian_elliptic = s21cac
nag_jacobian_elliptic = s21cbc
nag_jacobian_theta = s21ccc
nag_general_elliptic_integral_f = s21dac
nag_legendre_p = s22aac
nag_specfun_1f1_real = s22bac
nag_specfun_1f1_real_scaled = s22bbc
nag_specfun_2f1_real = s22bec
nag_specfun_2f1_real_scaled = s22bfc
nag_bsm_price = s30aac
nag_bsm_greeks = s30abc
nag_lookback_fls_price = s30bac
nag_lookback_fls_greeks = s30bbc
nag_binary_con_price = s30cac
nag_binary_con_greeks = s30cbc
nag_binary_aon_price = s30ccc
nag_binary_aon_greeks = s30cdc
nag_barrier_std_price = s30fac
nag_jumpdiff_merton_price = s30jac
nag_jumpdiff_merton_greeks = s30jbc
nag_heston_price = s30nac
nag_heston_greeks = s30nbc
nag_heston_term = s30ncc
nag_amer_bs_price = s30qcc
nag_asian_geom_price = s30sac
nag_asian_geom_greeks = s30sbc
__all__ = ['nag_airy_ai', 'nag_airy_ai_deriv',
           'nag_airy_ai_deriv_vector', 'nag_airy_ai_vector',
           'nag_airy_bi', 'nag_airy_bi_deriv',
           'nag_airy_bi_deriv_vector', 'nag_airy_bi_vector',
           'nag_amer_bs_price', 'nag_arccosh', 'nag_arcsinh',
           'nag_arctanh', 'nag_asian_geom_greeks',
           'nag_asian_geom_price', 'nag_barrier_std_price',
           'nag_bessel_i0', 'nag_bessel_i0_scaled',
           'nag_bessel_i0_scaled_vector', 'nag_bessel_i0_vector',
           'nag_bessel_i1', 'nag_bessel_i1_scaled',
           'nag_bessel_i1_scaled_vector', 'nag_bessel_i1_vector',
           'nag_bessel_i_alpha', 'nag_bessel_i_nu',
           'nag_bessel_i_nu_scaled', 'nag_bessel_j0',
           'nag_bessel_j0_vector', 'nag_bessel_j1',
           'nag_bessel_j1_vector', 'nag_bessel_j_alpha',
           'nag_bessel_k0', 'nag_bessel_k0_scaled',
           'nag_bessel_k0_scaled_vector', 'nag_bessel_k0_vector',
           'nag_bessel_k1', 'nag_bessel_k1_scaled',
           'nag_bessel_k1_scaled_vector', 'nag_bessel_k1_vector',
           'nag_bessel_k_alpha', 'nag_bessel_k_alpha_scaled',
           'nag_bessel_k_nu', 'nag_bessel_k_nu_scaled',
           'nag_bessel_y0', 'nag_bessel_y0_vector', 'nag_bessel_y1',
           'nag_bessel_y1_vector', 'nag_bessel_zeros',
           'nag_binary_aon_greeks', 'nag_binary_aon_price',
           'nag_binary_con_greeks', 'nag_binary_con_price',
           'nag_bsm_greeks', 'nag_bsm_price', 'nag_complex_airy_ai',
           'nag_complex_airy_bi', 'nag_complex_bessel_i',
           'nag_complex_bessel_j', 'nag_complex_bessel_j_seq',
           'nag_complex_bessel_k', 'nag_complex_bessel_y',
           'nag_complex_erfc', 'nag_complex_hankel',
           'nag_complex_log_gamma', 'nag_complex_polygamma',
           'nag_cos_integral', 'nag_cosh', 'nag_cumul_normal',
           'nag_cumul_normal_complem', 'nag_dawson',
           'nag_elliptic_integral_E', 'nag_elliptic_integral_F',
           'nag_elliptic_integral_complete_E',
           'nag_elliptic_integral_complete_K',
           'nag_elliptic_integral_pi', 'nag_elliptic_integral_rc',
           'nag_elliptic_integral_rd', 'nag_elliptic_integral_rf',
           'nag_elliptic_integral_rj', 'nag_erf', 'nag_erfc',
           'nag_erfcx', 'nag_exp_integral', 'nag_fresnel_c',
           'nag_fresnel_c_vector', 'nag_fresnel_s',
           'nag_fresnel_s_vector', 'nag_gamma',
           'nag_general_elliptic_integral_f', 'nag_heston_greeks',
           'nag_heston_price', 'nag_heston_term',
           'nag_incomplete_beta', 'nag_incomplete_gamma',
           'nag_jacobian_elliptic', 'nag_jacobian_theta',
           'nag_jumpdiff_merton_greeks', 'nag_jumpdiff_merton_price',
           'nag_kelvin_bei', 'nag_kelvin_bei_vector',
           'nag_kelvin_ber', 'nag_kelvin_ber_vector',
           'nag_kelvin_kei', 'nag_kelvin_kei_vector',
           'nag_kelvin_ker', 'nag_kelvin_ker_vector',
           'nag_legendre_p', 'nag_log_beta', 'nag_log_gamma',
           'nag_lookback_fls_greeks', 'nag_lookback_fls_price',
           'nag_polygamma_deriv', 'nag_polygamma_fun',
           'nag_real_jacobian_elliptic', 'nag_real_polygamma',
           'nag_scaled_log_gamma', 'nag_shifted_log',
           'nag_sin_integral', 'nag_sinh', 'nag_specfun_1f1_real',
           'nag_specfun_1f1_real_scaled', 'nag_specfun_2f1_real',
           'nag_specfun_2f1_real_scaled', 'nag_tanh', 's01bac',
           's10aac', 's10abc', 's10acc', 's11aac', 's11abc', 's11acc',
           's13aac', 's13acc', 's13adc', 's14aac', 's14abc', 's14acc',
           's14adc', 's14aec', 's14afc', 's14agc', 's14ahc', 's14bac',
           's14cbc', 's14ccc', 's15abc', 's15acc', 's15adc', 's15aec',
           's15afc', 's15agc', 's15ddc', 's17acc', 's17adc', 's17aec',
           's17afc', 's17agc', 's17ahc', 's17ajc', 's17akc', 's17alc',
           's17aqc', 's17arc', 's17asc', 's17atc', 's17auc', 's17avc',
           's17awc', 's17axc', 's17dcc', 's17dec', 's17dgc', 's17dhc',
           's17dlc', 's18acc', 's18adc', 's18aec', 's18afc', 's18aqc',
           's18arc', 's18asc', 's18atc', 's18ccc', 's18cdc', 's18cec',
           's18cfc', 's18cqc', 's18crc', 's18csc', 's18ctc', 's18dcc',
           's18dec', 's18ecc', 's18edc', 's18eec', 's18efc', 's18egc',
           's18ehc', 's18ejc', 's18ekc', 's18gkc', 's19aac', 's19abc',
           's19acc', 's19adc', 's19anc', 's19apc', 's19aqc', 's19arc',
           's20acc', 's20adc', 's20aqc', 's20arc', 's21bac', 's21bbc',
           's21bcc', 's21bdc', 's21bec', 's21bfc', 's21bgc', 's21bhc',
           's21bjc', 's21cac', 's21cbc', 's21ccc', 's21dac', 's22aac',
           's22bac', 's22bbc', 's22bec', 's22bfc', 's30aac', 's30abc',
           's30bac', 's30bbc', 's30cac', 's30cbc', 's30ccc', 's30cdc',
           's30fac', 's30jac', 's30jbc', 's30nac', 's30nbc', 's30ncc',
           's30qcc', 's30sac', 's30sbc']
