"""
Deprecated wrappers for the NAG C Library (Mark 26.0) x06 Chapter.

x06 - OpenMP Utilities.

https://www.nag.com/numeric/cl/nagdoc_cl26/html/x06/x06conts.html

NAG Copyright 2017.
"""

from ctypes import POINTER
from .util import get_library
from .util import Integer
from .util import NagError

_library_cl = get_library()

x06aac_ctypes = _library_cl.x06aac
x06aac_ctypes.restype = None
x06aac_ctypes.argtypes = [Integer, POINTER(NagError)]

def x06aac(num, fail):
    """
    Sets the number of threads for OpenMP parallel regions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x06/x06aac.html

    """
    x06aac_ctypes(num, fail)

nag_omp_set_num_threads = x06aac

x06abc_ctypes = _library_cl.x06abc
x06abc_ctypes.restype = Integer
x06abc_ctypes.argtypes = []

def x06abc():
    """
    The number of OpenMP threads in the current team.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x06/x06abc.html

    """
    return_val = x06abc_ctypes()
    return return_val

nag_omp_get_num_threads = x06abc

x06acc_ctypes = _library_cl.x06acc
x06acc_ctypes.restype = Integer
x06acc_ctypes.argtypes = []

def x06acc():
    """
    An upper bound on the number of threads in the next parallel region.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x06/x06acc.html

    """
    return_val = x06acc_ctypes()
    return return_val

nag_omp_get_max_threads = x06acc

x06adc_ctypes = _library_cl.x06adc
x06adc_ctypes.restype = Integer
x06adc_ctypes.argtypes = []

def x06adc():
    """
    The OpenMP thread number of the calling thread.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x06/x06adc.html

    """
    return_val = x06adc_ctypes()
    return return_val

nag_omp_get_thread_num = x06adc

x06afc_ctypes = _library_cl.x06afc
x06afc_ctypes.restype = Integer
x06afc_ctypes.argtypes = []

def x06afc():
    """
    Tests for an active OpenMP parallel region.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x06/x06afc.html

    """
    return_val = x06afc_ctypes()
    return return_val

nag_omp_in_parallel = x06afc

x06agc_ctypes = _library_cl.x06agc
x06agc_ctypes.restype = None
x06agc_ctypes.argtypes = [Integer]

def x06agc(nesting):
    """
    Enables or disables nested OpenMP parallelism.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x06/x06agc.html

    """
    x06agc_ctypes(nesting)

nag_omp_set_nested = x06agc

x06ahc_ctypes = _library_cl.x06ahc
x06ahc_ctypes.restype = Integer
x06ahc_ctypes.argtypes = []

def x06ahc():
    """
    Tests the status of nested OpenMP parallelism.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x06/x06ahc.html

    """
    return_val = x06ahc_ctypes()
    return return_val

nag_omp_get_nested = x06ahc

x06xac_ctypes = _library_cl.x06xac
x06xac_ctypes.restype = Integer
x06xac_ctypes.argtypes = []


def x06xac():
    """
    Tests whether a threaded NAG Library is being used.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x06/x06xac.html

    """
    return_val = x06xac_ctypes()
    return return_val

nag_using_threaded_impl = x06xac

__all__ = ['x06aac', 'x06abc', 'x06acc', 'x06adc', 'x06afc', 'x06agc', 'x06ahc',
           'x06xac',
           'nag_omp_set_num_threads', 'nag_omp_get_num_threads',
           'nag_omp_get_max_threads', 'nag_omp_get_thread_num',
           'nag_omp_in_parallel', 'nag_omp_set_nested', 'nag_omp_get_nested',
           'nag_using_threaded_impl']
