#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the anova submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018-2019.

# pylint: disable=invalid-name

import unittest
import warnings

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestG04ca(IntegrationTest):
    """
    Abstract suite of integration tests for the g04ca wrapper.
    """

    # pylint: disable=too-few-public-methods
    y = [
        274., 361., 253., 325., 317., 339., 326., 402., 336., 379., 345.,
        361., 352., 334., 318., 339., 393., 358., 350., 340., 203., 397.,
        356., 298., 382., 376., 355., 418., 387., 379., 432., 339., 293.,
        322., 417., 342., 82., 297., 133., 306., 352., 361., 220., 333., 270.,
        388., 379., 274., 336., 307., 266., 389., 333., 353,
    ]

class TestG04caNp(TestG04ca, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import anova as chapter
        TestG04ca.set_up(self)
        self.fun = chapter.factorial
        self.args = [
            self.y, # y, 0
            [6, 3], # lfac, 1
            3, # nblock, 2
            2, # inter, 3
            0, # irdf, 4
            42, # mterm, 5
            42, # maxt, 6
        ]

    def test_small_mterm(self):
        """
        mterm too small.
        """
        self.args[5] = 0
        warnings.simplefilter('ignore', b_utils.NagAlgorithmicWarning)
        res = self.fun(*self.args)
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)
        self.assertEqual(res.itotal, 6)

    def test_small_maxt(self):
        """
        maxt too small.
        """
        self.args[6] = 0
        warnings.simplefilter('ignore', b_utils.NagAlgorithmicWarning)
        res = self.fun(*self.args)
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)
        self.assertEqual(res.imean[0], 27)

if __name__ == '__main__': # pragma: no cover
    unittest.main()
