#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the linsys submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2019.

# pylint: disable=invalid-name

import unittest
import warnings

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestF04ba(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f04ba wrapper.
    """

class TestF04baBase(TestF04ba, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import linsys as chapter
        TestF04ba.set_up(self)
        self.fun = chapter.real_square_solve

    def test_singular(self):
        """
        Singular matrix call.
        """
        n = 2
        a = [[3., 6.], [1., 2.]]
        b = [[1.], [1.]]
        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)
        with self.assertRaises(b_utils.NagAlgorithmicWarning) as w_ctxt:
            self.fun(n, a, b)
        self.assertIn(
            w_ctxt.exception._errcodes, # pylint: disable=protected-access
            [(n, n, 5), (n+1, n+1, 6)],
        )
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)

if __name__ == '__main__': # pragma: no cover
    unittest.main()
