#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the quad submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2019.

# pylint: disable=invalid-name

import math
import unittest

import numpy as np

from test_functionality_core import IntegrationTest

class TestD01ra(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the d01ra wrapper.
    """

class TestD01raNp(TestD01ra, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import quad as chapter
        TestD01ra.set_up(self)
        self.opt_set = chapter.opt_set
        self.fun = chapter.dim1_gen_vec_multi_rcomm
        self.dimreq = chapter.dim1_gen_vec_multi_dimreq

    def test_good(self):
        """
        Good call.
        """
        # pylint: disable=too-many-locals
        ni = 2
        irevcm = 1
        a = 0.
        b = math.pi
        needi = np.zeros(ni, dtype=int)
        nx = -42
        dinest = np.empty(ni)
        errest = np.empty(ni)
        nagcal = -42

        comm = {}
        for optstr in (
                'Initialize = dim1_gen_vec_multi_rcomm',
                'Quadrature Rule = gk41',
                'Absolute Tolerance = 1.0e-7',
                'Relative Tolerance = 1.0e-7',
        ):
            self.opt_set(optstr, comm)

        dims = self.dimreq(ni, comm['iopts'], comm['opts'])

        x = np.empty(dims.lenxrq)
        fm = np.empty((dims.ldfmrq, dims.sdfmrq))

        while True:
            irevcm, _, nx = self.fun(
                irevcm, a, b, needi, x, nx, fm, dinest, errest, comm, nagcal,
            )

            if irevcm == 0:
                break

            fm[1, :nx] = x[:nx]*np.sin(2.*x[:nx])
            fm[0, :nx] = fm[1, :nx]*np.cos(15.*x[:nx])
            fm[1, :nx] = fm[1, :nx]*x[:nx]*np.cos(50.*x[:nx])

        np.testing.assert_allclose(
            dinest,
            [-0.0284307, 0.00790834],
            rtol=1.e-6,
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
